"""Run all demos."""

# Copyright (C) 2008-2014 Ilmar Wilbers
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Anders Logg, 2008-2009.
# Modified by Johannes Ring, 2009, 2011-2012.
# Modified by Johan Hake, 2009.
# Modified by Benjamin Kehlet 2012
# Modified by Martin Sandve Alnaes, 2014

from __future__ import print_function
import sys, os, re
import platform
import instant
from time import time
from instant import get_status_output
from dolfin import has_mpi, has_parmetis, has_scotch


def get_executable_name(demo, lang) :
  """Extract name of executable (without extension) from path.
     Name should be on the form demo_dir1_dir2 where dir1 and dir2 are
     directories under demo/[undocumented,documented] and lang (cpp or py)
     should be excluded.
  """
  directories = demo.split(os.path.sep)

  # Search for "demo" and lang from right
  directories_reverted = directories[::-1]
  demo_index = len(directories) - directories_reverted.index("demo") - 1
  lang_index = len(directories) - directories_reverted.index(lang) - 1
  truncated_directories = directories[demo_index+2:lang_index]
  return 'demo_' + "_".join(truncated_directories)

def run_cpp_demo(prefix, demo, rootdir, timing, failed):
    print("----------------------------------------------------------------------")
    print("Running C++ demo %s%s" % (prefix, demo))
    print("")

    cppdemo_executable = get_executable_name(demo, "cpp")
    if platform.system() == 'Windows':
        cppdemo_executable += '.exe'

    t1 = time()
    os.chdir(demo)
    status, output = get_status_output("%s .%s%s" % (prefix, os.path.sep,
                                                     cppdemo_executable))
    os.chdir(rootdir)
    t2 = time()
    timing += [(t2 - t1, demo)]

    if status == 0:
        print("OK")
    elif status == 10: # Failing but exiting gracefully
        print("ok (graceful exit on fail)")
    else:
        print("*** Failed")
        print(output)
        failed += [(demo, "C++", prefix, output)]

def run_python_demo(prefix, demo, rootdir, timing, failed):
    print("----------------------------------------------------------------------")
    print("Running Python demo %s%s" % (prefix, demo))
    print("")

    demofile = get_executable_name(demo, "python") + '.py'

    t1 = time()
    os.chdir(demo)
    status, output = get_status_output("%s %s -u %s" % (prefix, sys.executable,
                                                        demofile))
    os.chdir(rootdir)
    t2 = time()
    timing += [(t2 - t1, demo)]

    if status == 0:
        print("OK")
    elif status == 10: # Failing but exiting gracefully
        print("ok (graceful exit on fail)")
    else:
        print("*** Failed")
        print(output)

        # Add contents from Instant's compile.log to output
        instant_compile_log = os.path.join(instant.get_default_error_dir(),
                                           "compile.log")
        if os.path.isfile(instant_compile_log):
            instant_error = file(instant_compile_log).read()
            output += "\n\nInstant compile.log for %s:\n\n" % demo
            output += instant_error
        failed += [(demo, "Python", prefix, output)]

def main():
    # Location of all demos
    demodir = os.path.join(os.curdir, "..", "..", "demo")
    rootdir = os.path.abspath(os.curdir)

    # List of demos that have demo dir but are not currently implemented
    # NOTE: Demo must be listed here iff unimplemented otherwse the test will
    #       fail. This is meant to protect against usual bad named demos not
    #       executed for ages in regression tests.
    not_implemented = \
      [os.path.join(demodir, 'undocumented', 'projection-interpolation',    'cpp'),
       os.path.join(demodir, 'undocumented', 'interpolation',               'cpp'),
       os.path.join(demodir, 'undocumented', 'gmg-poisson',                 'python'),
       os.path.join(demodir, 'undocumented', 'gmg-stokes',                  'python'),
       os.path.join(demodir, 'undocumented', 'adaptive-poisson',            'cpp'),
       os.path.join(demodir, 'undocumented', 'multistage-solver',           'cpp'),
       os.path.join(demodir, 'undocumented', 'smoothing',                   'cpp'),
       os.path.join(demodir, 'undocumented', 'overlapping-regions',         'cpp'),
       os.path.join(demodir, 'undocumented', 'sub-function-assignment',     'cpp'),
       os.path.join(demodir, 'undocumented', 'compiled-extension-module',   'cpp'),
       os.path.join(demodir, 'undocumented', 'timing',                      'cpp'),
       os.path.join(demodir, 'undocumented', 'mplot',                       'cpp'),
       os.path.join(demodir, 'undocumented', 'plot',                        'cpp'),
       os.path.join(demodir, 'undocumented', 'coordinates',                 'cpp'),
       os.path.join(demodir, 'undocumented', 'multimesh-quadrature',        'cpp'),
       os.path.join(demodir, 'undocumented', 'multimesh-3d',                'cpp'),       
       os.path.join(demodir, 'documented',   'stokes-mini',                 'cpp'),
       os.path.join(demodir, 'documented',   'tensor-weighted-poisson',     'cpp'),
       os.path.join(demodir, 'documented',   'subdomains-poisson',          'cpp'),
       os.path.join(demodir, 'documented',   'singular-poisson-rst',        'cpp'),
       os.path.join(demodir, 'documented',   'maxwell-eigenvalues',        'cpp'),
       ]

    # Check if we should run only Python tests, use for quick testing
    if (len(sys.argv) == 2 and sys.argv[1] == "--only-python") or \
       "DISABLE_CPP_TESTING" in os.environ:
        only_python = True
    else:
        only_python = False

    # Check if we should run only C++ tests
    if (len(sys.argv) == 2 and sys.argv[1] == "--only-cpp") or \
       "DISABLE_PYTHON_TESTING" in os.environ:
        only_cpp = True
    else:
        only_cpp = False

    # Demos to run
    cppdemos = []
    pydemos = []
    for dpath, dnames, fnames in os.walk(demodir):
        if os.path.basename(dpath) == 'cpp' and not only_python:
            if os.path.isfile(os.path.join(dpath, 'Makefile')):
                cppdemos.append(dpath)
                assert not dpath in not_implemented, \
                    "Demo '%s' marked as not_implemented" % dpath
            else:
                assert dpath in not_implemented, \
                    "Non-existing demo '%s' not marked as not_implemented" % dpath
        elif os.path.basename(dpath) == 'python' and not only_cpp:
            tmp = dpath.split(os.path.sep)[-2]
            if os.path.isfile(os.path.join(dpath, 'demo_' + tmp + '.py')):
                pydemos.append(dpath)
                assert not dpath in not_implemented, \
                    "Demo '%s' marked as not_implemented" % dpath
            else:
                assert dpath in not_implemented, \
                    "Non-existing demo '%s' not marked as not_implemented" % dpath

    # Set non-interactive
    os.putenv('MPLBACKEND', 'agg')

    print("Running all demos (non-interactively)")
    print("")
    print("Found %d C++ demos" % len(cppdemos))
    print("Found %d Python demos" % len(pydemos))
    print("")
    import pprint

    # Push slow demos to the end
    pyslow = []
    cppslow = []
    for s in pyslow:
        if s in pydemos:
            pydemos.remove(s)
            pydemos.append(s)
    for s in cppslow:
        if s in cppdemos:
            cppdemos.remove(s)
            cppdemos.append(s)

    # List of demos that throw expected errors in parallel
    not_working_in_parallel = \
      [os.path.join(demodir, 'undocumented', 'adaptive-poisson',            'python'),
       os.path.join(demodir, 'undocumented', 'auto-adaptive-navier-stokes', 'cpp'),
       os.path.join(demodir, 'undocumented', 'auto-adaptive-navier-stokes', 'python'),
       os.path.join(demodir, 'documented',   'auto-adaptive-poisson',       'cpp'),
       os.path.join(demodir, 'documented',   'auto-adaptive-poisson',       'python'),
       os.path.join(demodir, 'undocumented', 'eval',                        'cpp'),
       os.path.join(demodir, 'undocumented', 'eval',                        'python'),
       os.path.join(demodir, 'undocumented', 'extrapolation',               'cpp'),
       os.path.join(demodir, 'undocumented', 'extrapolation',               'python'),
       os.path.join(demodir, 'undocumented', 'meshfunction-refinement',     'cpp'),
       os.path.join(demodir, 'documented',   'nonmatching-interpolation',   'cpp'),
       os.path.join(demodir, 'documented',   'nonmatching-interpolation',   'python'),
       os.path.join(demodir, 'undocumented', 'nonmatching-projection',      'cpp'),
       os.path.join(demodir, 'undocumented', 'nonmatching-projection',      'python'),
       os.path.join(demodir, 'undocumented', 'poisson-disc',                'python'),
       os.path.join(demodir, 'undocumented', 'poisson-disc',                'cpp'),
       os.path.join(demodir, 'undocumented', 'smoothing',                   'python'),
       os.path.join(demodir, 'documented',   'subdomains',                  'cpp'),
       os.path.join(demodir, 'documented',   'subdomains',                  'python'),
       os.path.join(demodir, 'undocumented', 'submesh',                     'cpp'),
       os.path.join(demodir, 'undocumented', 'submesh',                     'python'),
       os.path.join(demodir, 'undocumented', 'time-series',                 'cpp'),
       os.path.join(demodir, 'undocumented', 'time-series',                 'python'),
       os.path.join(demodir, 'undocumented', 'triangulate',                 'cpp'),
       os.path.join(demodir, 'undocumented', 'triangulate',                 'python'),
       os.path.join(demodir, 'undocumented', 'poisson1D-in-2D',             'cpp'),
       os.path.join(demodir, 'undocumented', 'poisson1D-in-2D',             'python'),
       os.path.join(demodir, 'undocumented', 'compiled-extension-module',   'python'),
       os.path.join(demodir, 'undocumented', 'coordinates',                 'python'),
       ]

    failed = []
    timing = []

    # Check if we should skip C++ demos
    if only_python:
        print("Skipping C++ demos")
        cppdemos = []

    # Check if we should skip Python demos
    if only_cpp:
        print("Skipping Python demos")
        pydemos = []

    # Build prefix list
    prefixes = [""]
    mpi_prefix = "mpirun -np 3 "
    if has_mpi() and (has_parmetis() or has_scotch()):
        prefixes.append(mpi_prefix)
    else:
        print("Not running regression tests in parallel.")

    # Allow to disable parallel testing
    if "DISABLE_PARALLEL_TESTING" in os.environ:
        print("Skip running demos in parallel")
        prefixes = [""]

    # Allow to disable serial testing
    if "DISABLE_SERIAL_TESTING" in os.environ:
        print("Skip running demos in serial")
        prefixes.remove("")

    # Run in serial, then in parallel
    for prefix in prefixes:

        # List of demos to run
        if prefix == mpi_prefix:
            cppdemos_to_run = list(set(cppdemos) - set(not_working_in_parallel))
            pydemos_to_run  = list(set(pydemos)  - set(not_working_in_parallel))
        else:
            cppdemos_to_run = cppdemos
            pydemos_to_run  = pydemos

        # Run demos
        for demo in cppdemos_to_run:
            run_cpp_demo(prefix, demo, rootdir, timing, failed)
        for demo in pydemos_to_run:
            run_python_demo(prefix, demo, rootdir, timing, failed)

    # Print summary of time to run demos
    timing.sort()
    print("")
    print("Time to run demos:")
    print("\n".join("%.2fs: %s" % t for t in timing))

    total_no_demos = len(pydemos)
    if not only_python:
        total_no_demos += len(cppdemos)

    # Print output for failed tests
    print("")
    if len(failed) > 0:
        print("%d demo(s) out of %d failed, see demo.log for details." %
              (len(failed), total_no_demos))
        file = open("demo.log", "w")
        for (test, interface, prefix, output) in failed:
            file.write("----------------------------------------------------------------------\n")
            file.write("%s%s (%s)\n" % (prefix, test, interface))
            file.write("\n")
            file.write(output)
            file.write("\n")
            file.write("\n")
    else:
        print("All demos checked: OK")

    # Return error code if tests failed
    return len(failed)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
#
# Read doxygen xml files to find all members of the dolfin
# name space and generate API doc files per subdirectory of
# dolfin
#
# Written by Tormod Landet, 2017
#
from __future__ import print_function
import sys, os
import parse_doxygen


DOXYGEN_XML_DIR = 'doxygen/xml'
API_GEN_DIR = 'generated_rst_files'
SWIG_DIR = '../dolfin/swig/'
SWIG_FILE = 'docstrings.i'
MOCK_PY = 'mock_cpp_modules.py'


def get_subdir(hpp_file_name):
    """
    Return "subdir" for a path name like
        /path/to/dolfin/subdir/a_header.h
    """
    path_components = hpp_file_name.split(os.sep)
    path_components_rev = path_components[::-1]
    idx = path_components_rev.index('dolfin')
    subdir = path_components_rev[idx - 1]
    return subdir


def get_short_path(hpp_file_name):
    """
    Return "dolfin/subdir/a_header.h" for a path name like
        /path/to/dolfin/subdir/a_header.h
    """
    path_components = hpp_file_name.split(os.sep)

    if 'dolfin' in path_components:
        # dolfin header files
        path_components_rev = path_components[::-1]
        idx = path_components_rev.index('dolfin')
        short_path = path_components_rev[:idx + 1]
    else:
        # ufc header files
        short_path = path_components[-1:]
    return os.sep.join(short_path[::-1])


def write_rst(subdir, subdir_members, api_gen_dir):
    """
    Write files for Sphinx C++ API documentation
    """
    rst_name = os.path.join(api_gen_dir, 'api_gen_%s.rst' % subdir)
    print('Generating', rst_name)

    # Make output directory
    if not os.path.isdir(api_gen_dir):
        os.mkdir(api_gen_dir)

    prev_short_name = ''
    with open(rst_name, 'wt') as rst:
        rst.write('.. automatically generated by generate_api_rst.py and parse_doxygen.py\n')
        #rst.write('dolfin/%s\n%s' % (subdir, '=' * 80))
        #rst.write('\nDocumentation for C++ code found in dolfin/%s/*.h\n\n' % subdir)
        rst.write('\n.. contents::\n\n\n')

        kinds = [('typedef', 'Type definitions', 'doxygentypedef'),
                 ('enum', 'Enumerations', 'doxygenenum'),
                 ('function', 'Functions', 'doxygenfunction'),
                 ('struct', 'Structures', 'doxygenstruct'),
                 ('variable', 'Variables', 'doxygenvariable'),
                 ('class', 'Classes', 'doxygenclass')]

        for kind, kind_name, directive in kinds:
            if kind in subdir_members:
                # Write header H2
                rst.write('%s\n%s\n\n' % (kind_name, '-'*70))

                for name, member in sorted(subdir_members[kind].items()):
                    short_name = member.short_name
                    fn = get_short_path(member.hpp_file_name)

                    # Write header H3
                    if short_name != prev_short_name:
                        rst.write('%s\n%s\n\n' % (short_name, '~'*60))
                    prev_short_name = short_name

                    # Info about filename
                    rst.write('C++ documentation for ``%s`` from ``%s``:\n\n' % (short_name, fn))

                    # Write documentation for this item
                    rst.write(member.to_rst())
                    rst.write('\n\n')


def write_swig(subdir, subdir_members, swig_dir, swig_file_name, swig_header=''):
    """
    Write files for SWIG so that we get docstrings in Python
    """
    swig_subdir = os.path.join(swig_dir, subdir)
    if not os.path.isdir(swig_subdir):
        os.mkdir(swig_subdir)

    swig_iface_name = os.path.join(swig_subdir, swig_file_name)
    print('Generating', swig_iface_name)

    with open(swig_iface_name, 'wt') as out:
        out.write(swig_header)
        out.write('// SWIG docstrings generated by doxygen and generate_api_rst.py / parse_doxygen.py\n\n')

        for kind in subdir_members:
            for name, member in sorted(subdir_members[kind].items()):
                out.write(member.to_swig())
                out.write('\n')


def write_mock_modules(namespace_members, mock_py_module):
    """
    Write a mock module so that we can create documentation for
    dolfin on ReadTheDocs where we cannot compile so that the
    dolfin.cpp.* module are not available. We fake those, but
    include the correct docstrings
    """
    print('Generating', mock_py_module)

    mydir = os.path.dirname(os.path.abspath(__file__))
    swig_module_dir = os.path.join(mydir, '..', 'dolfin', 'swig', 'modules')
    swig_module_dir = os.path.abspath(swig_module_dir)

    if not os.path.isdir(swig_module_dir):
        print('SWIG module directory is not present,', swig_module_dir)
        print('No mock Python code will be generated')
        return

    with open(mock_py_module, 'wt') as out:
        out.write('#!/usr/bin/env python\n')
        out.write('#\n')
        out.write('# This file is AUTO GENERATED!\n')
        out.write('# This file is fake, full of mock stubs\n')
        out.write('# This file is made by generate_api_rst.py\n')
        out.write('#\n\n')
        out.write('from __future__ import print_function\n')
        out.write('from types import ModuleType\n')
        out.write('import sys\n')
        out.write('\n\nWARNING = "This is a mock object!"\n')

        # Loop over SWIG modules and generate mock Python modules
        for module_name in os.listdir(swig_module_dir):
            module_i = os.path.join(swig_module_dir, module_name, 'module.i')
            if not os.path.isfile(module_i):
                continue

            # Find out which headers are included in this SWIG module
            included_headers = set()
            for line in open(module_i):
                if line.startswith('#include'):
                    header = line[8:].strip()[1:-1]
                    included_headers.add(header)
                elif line.startswith('%import'):
                    header = line.split(')')[1].strip()[1:-1]
                    included_headers.add(header)

            module_py_name = '_' + module_name
            full_module_py_name = 'dolfin.cpp.' + module_py_name
            out.write('\n\n' + '#'*80 + '\n')
            out.write('%s = ModuleType("%s")\n' % (module_py_name, full_module_py_name))
            out.write('sys.modules["%s"] = %s\n' % (full_module_py_name, module_py_name))
            out.write('\n')
            print('    Generating module', full_module_py_name)

            for member in namespace_members:
                # Check if this member is included in the given SWIG module
                hpp_file_name = get_short_path(member.hpp_file_name)
                if hpp_file_name not in included_headers:
                    continue

                out.write(member.to_mock(modulename=module_py_name))
                out.write('\n\n')


def parse_doxygen_xml_and_generate_rst_and_swig(xml_dir, api_gen_dir, swig_dir, swig_file_name,
                                                swig_header='', mock_py_module=''):
    # Read doxygen XML files and split namespace members into
    # groups based on subdir and kind (class, function, enum etc)
    create_subdir_groups_if_missing = False
    if os.path.isdir(xml_dir):
        namespaces = parse_doxygen.read_doxygen_xml_files(xml_dir, ['dolfin', 'ufc'])
    else:
        raise OSError('Missing doxygen XML directory %r' % xml_dir)

    # Group all documented members into subdir groups (io, la, mesh, fem etc)
    sorted_members = list(namespaces['dolfin'].members.values())
    sorted_members.sort(key=lambda m: m.name)
    all_members = {}
    for member in sorted_members:
        subdir = get_subdir(member.hpp_file_name)
        sd = all_members.setdefault(subdir, {})
        kd = sd.setdefault(member.kind, {})
        kd[member.name] = member

    # Generate Sphinx RST files and SWIG interface files
    for subdir, subdir_members in sorted(all_members.items()):
        if subdir:
            if api_gen_dir:
                write_rst(subdir, subdir_members, api_gen_dir)
            if swig_dir:
                write_swig(subdir, subdir_members, swig_dir, swig_file_name, swig_header)

    # Write UFC documenttation, no SWIG for UFC, only RST
    if api_gen_dir:
        ufc_members = {}
        for member in namespaces['ufc'].members.values():
            kd = ufc_members.setdefault(member.kind, {})
            kd[member.name] = member
        write_rst('ufc', ufc_members, api_gen_dir)

    # Generate a mock Python module
    if mock_py_module:
        write_mock_modules(sorted_members, mock_py_module)


if __name__ == '__main__':
    swig_dir = SWIG_DIR
    allow_empty_xml = False

    if '--no-swig' in sys.argv:
        swig_dir = None

    parse_doxygen_xml_and_generate_rst_and_swig(DOXYGEN_XML_DIR, API_GEN_DIR, swig_dir,
                                                SWIG_FILE, '', MOCK_PY)



# -*- coding: utf-8 -*-
# Copyright (C) 2009-2011 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Garth N. Wells, 2012
#
# First added:  2008-10-22
# Last changed: 2012-08-18

import types

from six import string_types

import dolfin.cpp as cpp
from dolfin.functions.constant import Constant
from dolfin.compilemodules.subdomains import CompiledSubDomain
import ufl
from dolfin.fem.projection import project

__all__ = ["AutoSubDomain", "DirichletBC"]


class AutoSubDomain(cpp.SubDomain):
    "Wrapper class for creating a SubDomain from an inside() function."

    def __init__(self, inside_function):
        "Create SubDomain subclass for given inside() function"

        # Check that we get a function
        if not isinstance(inside_function, types.FunctionType):
            cpp.dolfin_error("bcs.py",
                             "auto-create subdomain",
                             "Expecting a function (not %s)" %
                             str(type(inside_function)))
        self.inside_function = inside_function

        # Check the number of arguments
        if inside_function.__code__.co_argcount not in (1, 2):
            cpp.dolfin_error("bcs.py",
                             "auto-create subdomain",
                             "Expecting a function of the form inside(x) or inside(x, on_boundary)")
        self.num_args = inside_function.__code__.co_argcount

        cpp.SubDomain.__init__(self)

    def inside(self, x, on_boundary):
        "Return true for points inside the subdomain"

        if self.num_args == 1:
            return self.inside_function(x)
        else:
            return self.inside_function(x, on_boundary)


class DirichletBC(cpp.DirichletBC):

    # Reuse doc-string from cpp.DirichletBC
    __doc__ = cpp.DirichletBC.__doc__

    def __init__(self, *args, **kwargs):
        "Create Dirichlet boundary condition"

        # Copy constructor
        if len(args) == 1:
            if not isinstance(args[0], cpp.DirichletBC):
                cpp.dolfin_error("bcs.py",
                                 "create DirichletBC",
                                 "Expecting a DirichleBC as only argument"
                                 " for copy constructor")

            # Initialize base class
            cpp.DirichletBC.__init__(self, args[0])
            self.domain_args = args[0].domain_args
            return

        mpi_comm = args[0].mesh().mpi_comm()

        # Special case for value specified as float, tuple or similar
        if len(args) >= 2 and not isinstance(args[1], cpp.GenericFunction):
            if isinstance(args[1], ufl.classes.Expr):
                expr = project(args[1], args[0])
            else:
                expr = Constant(args[1])  # let Constant handle all problems
            args = args[:1] + (expr,) + args[2:]

        # Special case for sub domain specified as a function
        if len(args) >= 3 and isinstance(args[2], types.FunctionType):
            sub_domain = AutoSubDomain(args[2])
            args = args[:2] + (sub_domain,) + args[3:]

        # Special case for sub domain specified as a string
        if len(args) >= 3 and isinstance(args[2], string_types):
            sub_domain = CompiledSubDomain(args[2], mpi_comm=mpi_comm)
            args = args[:2] + (sub_domain,) + args[3:]

        # Store Expression to avoid scoping issue with SWIG directors
        if isinstance(args[1], cpp.Expression):
            self.function_arg = args[1]

        # Store SubDomain to avoid scoping issue with SWIG directors
        self.domain_args = args[2:]

        # FIXME: Handling of multiple default arguments does not
        # really work between Python and C++ when C++ requires them to
        # be ordered and cannot accept the latter without the
        # former...

        # Add keyword arguments (in correct order...)
        allowed_kwargs = ["method", "check_midpoint"]
        for key in allowed_kwargs:
            if key in kwargs:
                args = tuple(list(args) + [kwargs[key]])

        # Check for other keyword arguments
        for key in kwargs:
            if key not in allowed_kwargs:
                cpp.dolfin_error("bcs.py",
                                 "create boundary condition",
                                 "Unknown keyword argument \"%s\"" % key)

        # Initialize base class
        cpp.DirichletBC.__init__(self, *args)

    # Set doc string
    __init__.__doc__ = cpp.DirichletBC.__init__.__doc__



# -*- coding: utf-8 -*-
"""This module provides a simple way to compute the interpolation of a
:py:class:`Function <dolfin.functions.function.Function>` or
:py:class:`Expression <dolfin.functions.expression.Expression>` onto a
finite element space."""

# Copyright (C) 2009-2017 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Jørgen S. Dokken, 2017
# First added:  2009-08-29
# Last changed: 2017-03-17

# Local imports
import dolfin.cpp as cpp

from dolfin.functions.functionspace import FunctionSpace, MultiMeshFunctionSpace
from dolfin.functions.function import Function
from dolfin.functions.multimeshfunction import MultiMeshFunction

__all__ = ["interpolate"]


def interpolate(v, V):
    """Return interpolation of a given function into a given finite
    element space. Also supports MultiMesh.

    *Arguments*
        v
            a :py:class:`Function <dolfin.functions.function.Function>` or
            an :py:class:`Expression <dolfin.functions.expression.Expression>`
            or an :py:class`MultiMeshFunction <dolfin.functions.multimeshfunction.MultiMeshFunction>`
        V
            a :py:class:`FunctionSpace (standard, mixed, etc.)
            <dolfin.functions.functionspace.FunctionSpace>` or
            a :py:class:`MultiMeshFunctionSpace
            <dolfin.cpp.function.MultiMeshFunctionSpace>`.


    *Example of usage*

        .. code-block:: python

            v = Expression("sin(pi*x[0])")
            V = FunctionSpace(mesh, "Lagrange", 1)
            Iv = interpolate(v, V)

    """

    # Check arguments
    if not isinstance(V, (FunctionSpace, cpp.MultiMeshFunctionSpace)):
        cpp.dolfin_error("interpolation.py",
                         "compute interpolation",
                         "Illegal function space for interpolation, not a FunctionSpace (%s)" % str(v))
    if isinstance(V, cpp.MultiMeshFunctionSpace):
        Pv = MultiMeshFunction(V)
        Pv.interpolate(v)
        return Pv

    # Compute interpolation
    Pv = Function(V)
    Pv.interpolate(v)

    return Pv



# -*- coding: utf-8 -*-
"""Script to extract documentation from docstrings in *.h files in the DOLFIN
source tree."""

# Copyright (C) 2010 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Kristian B. Oelgaard, 2011.
# Modified by Marie E. Rognes, 2011.
# Modified by Anders E. Johansen, 2011.
#
# First added:  2010-08-26
# Last changed: 2011-07-10

import os, io

def extract_documentation(dolfin_dir, header, module):
    "Extract documentation for given header in given module"

    # print "Extracting documentation for %s..." % header

    # List of classes with documentation
    classnames = []
    documentation = []

    # Class name and parent class name
    classname = None
    parents = None

    # Comment and signature
    comment = None
    signature = None

    # Indentation of signatures
    indent = 0

    # Iterate over each line
    f = io.open(os.path.join(dolfin_dir, "dolfin", module, header), encoding="utf-8")
    for line in f:

        # Check for comment
        if "///" in line:

            # We may have either "///" and "/// "
            if "/// " in line:
                c = line.split("/// ")[1].rstrip()
            else:
                c = line.split("///")[1].rstrip()

            # Found start of new comment
            if comment is None:
                comment = c

            # Continuing comment on next line
            else:
                comment += "\n" + c

        # Check for class
        # If anything goes wrong, remove the last '<' after template.
        elif " class " in line and not ";" in line and not "//" in line and not "template<" in line:

            # Get class name and parent
            classname = line.split(" class ")[1].split(":")[0].strip()
            if "public" in line:
                # Strip of each parent of additional commas, blanks
                # and newlines
                parents = [p.strip(", \n") for p in line.split("public")[1:]]

                # Remove virtual modifier
                parents = [p.replace("virtual ", "") for p in parents]

            # Store documentation
            # TODO: KBO: we don't check if a given classname is in the dolfin
            # namepace.
            classnames.append(classname)
            documentation.append((classname, parents, comment, []))

            classname = None
            parents = None
            comment = None

        # If we encounter a '//' commented line we reset comment and proceed
        # This means that '///' and '//' should not be mixed when documenting
        # functions.
        elif line.lstrip()[0:2] == "//" and not line.lstrip()[0:3] == "///":
            comment = None
            continue

        # Check for function signature
        elif comment is not None:
            s = line.strip()

            # Found start of new signature
            if signature is None:
                signature = s
                #indent = (len(s.split("(")[0]) + 1)*" "

            # Continuing signature on next line
            else:
                #signature += "\n" + indent + s
                signature += " " + s

            # Signature ends when we find ";" or "{"
            if ";" in s or "{" in s:

                # Strip out last part
                signature = signature.split(";")[0]
                signature = signature.split("{")[0]
                signature = signature.strip()

                # Remove stuff Sphinx can't handle
                signature = signature.replace("virtual ", "")
                signature = signature.replace("inline ", "")

                # Remove ": stuff" for constructors
                new_s = []
                for l in signature.split("::"):
                    if not ":" in l:
                        new_s.append(l)
                    else:
                        new_s.append(l.split(":")[0])
                        break
                signature = "::".join(new_s).strip()

                # Remove template stuff (not handled correctly by Sphinx)
                # e.g., 'template <typename Kernel> CGAL::Bbox_3  bbox()' in mesh/Point.h
                if "template" in signature:
                    signature = ">".join(signature.split(">")[1:]).lstrip()

                # Only handle functions or enums, i.e. signatures that contain
                # '(' (or ')'). This is to avoid picking up data members.
                # NOTE, KBO: Should we also skip private functions?
                if not "(" in signature and not "enum" in signature:
                    # Reset comment and signature
                    comment = None
                    signature = None
                    continue

                # Skip destructors (not handled by Sphinx)
                destructor = "~" in signature

                # Get function name
                #function = signature.split("(")[0].split(" ")[-1]

                # Store documentation
                if len(documentation) > 0 and not destructor:
                    documentation[-1][-1].append((signature, comment))
                elif not destructor:
                    documentation = [(None, None, None, [(signature, comment)])]

                # Reset comment and signature
                comment = None
                signature = None

    # Close file
    f.close()

    # Sort documentation alphabetically within each class
#    for (classname, parent, comment, function_documentation) in documentation:
#        function_documentation.sort()

    return documentation, classnames

def extract_doc_representation(dolfin_dir):
    # Extract modules from dolfin.h
    modules = []
    f = open(os.path.join(dolfin_dir, "dolfin", "dolfin.h"))
    for line in f:
        if line.startswith("#include <dolfin/"):
            module = line.split("/")[1]
            modules += [module]
    f.close()

    # Iterate over modules
    documentation = {}
    classnames = []
    for module in modules:
#        if not module == "la":
#            continue
        # Extract header files from dolfin_foo.h
        f = open(os.path.join(dolfin_dir, "dolfin", module, "dolfin_%s.h" % module))
        documentation[module] = []
        for line in f:

            # Generate documentation for header file
            if line.startswith("#include <dolfin/"):
                header = line.split("/")[2].split(">")[0]
                # Skip version.h (generated from version.h.in via CMake)
                if header == "version.h":
                    continue
#                if not header == "GenericTensor.h":
#                    continue
                doc, cls = extract_documentation(dolfin_dir, header, module)
                documentation[module].append((header, doc))
                classnames += cls

    return documentation, classnames

if __name__ == "__main__":
    docs, classes = extract_doc_representation()
#    for c in classes:
#        print c
#    for key, doc in docs.items():
#        for header, cont in doc:
#            print cont






#!/usr/bin/env python
#
# Copyright (C) 2011 Marie E. Rognes
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Garth N. Wells, 2013
#
# Utility script for splitting the cpp and python demos into separate
# directory trees. Ignores cmake files for python.

from __future__ import print_function
import sys, os, shutil
import subprocess

index_template = """
Collection of documented demos
==============================

.. toctree::
   :glob:
   :numbered:
   :maxdepth: 1

   */*/%s/documentation

%s

.. note::

    You must have a working installation of FEniCS in order to run the
    demos.

"""


run_cpp_demos = """
To run the C++ demos, follow the below procedure:

* Download the source files i.e., ``main.cpp`` and ``CMakeLists.txt``,
  from the demo that you want to run. Some demos also contain UFL form
  files, e.g., ``Poisson.ufl``. Note that there may be multiple form
  files.

* Compile the form files to generate code with DOLFIN wrappers::

      $ ffc -l dolfin Poisson.ufl

  Generated .h files are usually distributed with the demos so you may
  choose to skip this step and use the provided header file directly,
  in this case ``Poisson.h``.

  If you wish to use optimized generated code, do::

      $ ffc -O -l dolfin Poisson.ufl

* Configure the demo build process::

      $ cmake .

* Compile the demo::

      $ make

* Run the created executable::

    $ ./demo

"""

run_python_demos = """
To run the Python demos, follow the below procedure:

* Download the source file, e.g., ``demo_poisson.py``, for the demo that you
  want to run.

* Use the Python interpreter to run this file::

      $ python demo.py

"""

run_instructions = {"cpp": run_cpp_demos, "python": run_python_demos}

def generate_main_index_file(output_dir, language):

    filename = os.path.join(output_dir, "index.rst")
    file = open(filename, "w")
    text = index_template % (language, run_instructions[language])
    file.write(text)
    file.close()


def copy_split_demo_doc(input_dir, cpp_output_dir, python_output_dir):

    # Get list of files in demo directories
    try:

        # Get root path of git repo
        git_root  = subprocess.check_output(["git", "rev-parse", "--show-toplevel"]).splitlines()

        # Get list of files tracked by git (relative to git root)
        git_files = subprocess.check_output(["git", "ls-files", "--full-name", input_dir]).splitlines()

        # Build list with full paths
        git_files = [os.path.join(git_root[0],  f) for f in git_files]

        if not git_files:
            # Workaround for when we're not in a git repo
            git_files = subprocess.check_output(["find", input_dir]).splitlines()
    except:
        git_files = None

    def ignore_cpp(directory, contents):
        if directory[-3:] == "cpp":
            return contents
        elif git_files is not None:
            return [c for c in contents if not in_git(directory, c, git_files, ["cpp"]) ]
        else:
            return []

    def ignore_python(directory, contents):
        if directory[-6:] == "python":
            return contents
        elif git_files is not None:
            return [c for c in contents if not in_git(directory, c, git_files, ["python"])]
        else:
            return []

    # Copy demo tree to cpp_output_dir ignoring python demos
    try:
        shutil.rmtree(cpp_output_dir)
    except:
        pass
    shutil.copytree(input_dir, cpp_output_dir, ignore=ignore_python)

    # In addition, generate main index file for navigating demos
    generate_main_index_file(cpp_output_dir, "cpp")

    # Copy demo tree to python_output_dir ignoring cpp demos
    try:
        shutil.rmtree(python_output_dir)
    except:
        pass
    shutil.copytree(input_dir, python_output_dir, ignore=ignore_cpp)

    # In addition, generate main index file for navigating demos
    generate_main_index_file(python_output_dir, "python")


def in_git(directory, name, git_files, exclude_dirs=[]):
    "Check whether file is version-controlled"

    # Make sure we have the real path (remove symlinks)
    directory = os.path.realpath(directory)

    # Get file full path
    f = os.path.join(directory, name)

    # Return true if a directory (git doesn't track directories)
    if os.path.isdir(f):
        if name in exclude_dirs:
            return False
        else:
            return True
    return f in git_files


if __name__ == "__main__":

    args = sys.argv[1:]

    if len(args) != 3:
        usage= "Usage: python copy_and_split_demo_doc.py input_dir cpp_output_dir python_output_dir"
        print(usage)
        sys.exit(2)

    copy_split_demo_doc(args[0], args[1], args[2])



#!/usr/bin/env python
#
# Copyright (C) 2011 Marie E. Rognes
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Utility script for creating main index.rst files in sphinx
# documentation projects in DOLFIN.

from __future__ import print_function
import os, sys

index_template = """

%(formatting)s
%(title)s
%(formatting)s

%(body)s

.. toctree::
   :hidden:

   programmers-reference/index
   demo/index
   quick_reference
"""

body = """
The Demos
=========

:doc:`The demos <demo/index>` (Collection of documented demo programs)

The Programmer's Reference
==========================

:ref:`Programmer's Reference Index <genindex>` (Classes, functions, terms)

"""
modules = """
DOLFIN modules
==================

:ref:`DOLFIN Module Index <modindex>` (Modules)

"""

bodies = {"C++": body, "Python": (body + modules)}

def generate_main_index_file(output_dir, language, version):

    filename = os.path.join(output_dir, "index.rst")
    file = open(filename, "w")
    title = "Documentation for DOLFIN-%s (%s)" % (version, language)
    text = index_template % {"title": title, "formatting": len(title)*"#",
                             "body": bodies[language]}
    file.write(text)
    file.close()

if __name__ == "__main__":

    args = sys.argv[1:]

    if len(args) != 3:
        usage= "Usage: python generate_main_index.py cpp_output_dir python_output_dir version"
        print(usage)
        sys.exit(2)

    generate_main_index_file(args[0], "C++", args[2])
    generate_main_index_file(args[1], "Python", args[2])





# -*- coding: utf-8 -*-
"""The multistage module of dolfin"""
from dolfin.multistage import multistagescheme
from dolfin.multistage import multistagesolvers
from dolfin.multistage import rushlarsenschemes

from .multistagescheme import *
from .multistagesolvers import *
from .rushlarsenschemes import *

# NOTE: The automatic documentation system in DOLFIN requires to _not_ define
# NOTE: classes or functions within this file. Use separate modules for that
# NOTE: purpose.

__all__ = multistagescheme.__all__ + multistagesolvers.__all__ + \
          rushlarsenschemes.__all__



# -*- coding: utf-8 -*-
# Utilities for parsing DOLFIN files and generaton of SWIG files
#
# Copyright (C) 2012 Johan Hake
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2012-07-01
# Last changed: 2012-07-03

# System imports
import os
import re

try:
    from collections import OrderedDict
except ImportError:
    from dolfin_utils.ordereddict import OrderedDict

# Local imports
from .CppHeaderParser import CppHeader

# reg exp pattern
_template_pattern = re.compile("\w+<([\w ]+)>")

def strip_templates(types):
    """
    Strip template types
    """
    ret_types = set()
    temp_types = re.findall(_template_pattern, types)
    while temp_types:
        ret_types.update(temp_types)
        for temp_type in temp_types:
            types = types.replace("<%s>" % temp_type, "")
        temp_types = re.findall(_template_pattern, types)

    if types:
        ret_types.add(types)

    return ret_types

def build_swig_import_info(dependencies, submodule_info, module_prepend="",
                           package_name="", parsed_modules=None):
    """
    Build import and include info from a list of submodules and headers
    """
    import_lines = []
    headers_includes = ["", "// Include types from dependent modules"]
    file_dependencies = []

    # Iterate over the file dependencies and generate import statements
    previous_submodule = ""
    for submodule, headers in list(dependencies.items()):
        module = submodule_info[submodule]["module"]
        headers_includes.append("")
        headers_includes.append("// #include types from %s submodule of "\
                                "module %s" % (submodule, module))
        import_lines.append("")
        import_lines.append("// %%import types from submodule"\
                            " %s of SWIG module %s" % \
                            (submodule, module))
        
        # Check for pre includes
        if submodule_info[submodule]["has_pre"]:
            file_dependencies.append("dolfin/swig/%s/pre.i" % submodule)
            import_lines.append(\
                "%%include \"dolfin/swig/%s/pre.i\"" % submodule)

        # Add headers
        file_dependencies.extend(headers)
        for header in headers:
            # Add file dependency
            if parsed_modules is None or module in parsed_modules:
                
                # If module is not parsed yet we introduce circular dependencies
                # in import statments which we want to avoid
                import_lines.append("%%import(package=\"%s\", module=\"%s\") \"%s\"" % \
                                    (package_name, module_prepend+module, header))
            headers_includes.append("#include \"%s\"" % header)

    return import_lines, headers_includes, file_dependencies

def parse_and_extract_type_info(code):
    """
    Parse header code and return declared types, used types, and any bases
    """
    
    used_types=set()
    declared_types=OrderedDict()

    # Use CppHeader to extract
    cppheader = CppHeader(code, argType="string")

    # Collect used types from the code
    used_types = set()

    # Iterate over typedefs and collect types
    for typedef in sorted(cppheader.typedefs):
        if "dolfin::" in typedef:
            typedef = typedef.replace("dolfin::", "")

        # Store type information
        declared_types[typedef] = []

    # Iterate over free functions and collect dependant types
    for function in cppheader.functions:
    
        # Check return type
        if function["rtnType"] != "void":
            used_types.update(strip_templates(function["rtnType"]))
    
        # Check argument types
        for argument in function["parameters"]:
            used_types.update(strip_templates(argument["type"]))

    # Iterate over class and collect info
    for class_name, class_repr in sorted(cppheader.classes.items()):

        # Check if class is private
        if class_repr["parent"] is not None:
            continue
    
        # Get bases
        bases = set()
        for base in class_repr["inherits"]:
            bases.update(strip_templates(base["class"]))

        # Remove itself from any bases
        bases.difference_update([class_name])

        # Register the class
        declared_types[class_name] = sorted(bases)
        
        # Iterate over public properties
        for prop in class_repr["properties"]["public"]:
            used_types.update(strip_templates(prop["type"]))

        # Iterate over methods and collect dependant types
        for method in class_repr["methods"]["public"]+\
                class_repr["methods"]["protected"]:
    
            # Check return type
            if method["rtnType"] != "void":
                used_types.update(strip_templates(method["rtnType"]))
    
            # Check argument types
            for argument in method["parameters"]:
                used_types.update(strip_templates(argument["type"]))
    
        # Remove any self dependencies
        used_types = set(used_type for used_type in
                         used_types if class_name not in used_type)

    return used_types, declared_types
    
def sort_submodule_dependencies(dependencies, submodule_info):
    """
    Given a dict of submodules and headers, we use original
    submodule info to sort the content
    """
    submodule_dependences = OrderedDict()
    for submodule in submodule_info:
        if submodule in dependencies:
            
            # Register sorted dependency
            submodule_dependences[submodule] = [header for header in \
                submodule_info[submodule]["headers"] \
                if header in dependencies[submodule]]
    
    return submodule_dependences




"This demo program demonstrates how to include additional C++ code in DOLFIN."

# Copyright (C) 2013 Kent-Andre Mardal, Mikael Mortensen, Johan Hake
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# First added:  2013-04-02

from __future__ import print_function
from dolfin import *
import numpy
import os

if has_pybind11():
    print("pybind11 support for compiled code is different")
    exit()

header_file = open("Probe/Probe.h")
code = "\n".join(header_file.readlines())
compiled_module = compile_extension_module(code=code, source_directory="Probe", \
                                           sources=["Probe.cpp"], \
                                           include_dirs=[".", os.path.abspath("Probe")])

mesh = UnitCubeMesh(10, 10, 10)
V = FunctionSpace(mesh, 'CG', 1)

x = numpy.array((0.5, 0.5, 0.5))
probe = compiled_module.Probe(x, V)

# Just create some random data to be used for probing
u0 = interpolate(Expression('x[0]', degree=1), V)
probe.eval(u0)
print("number of probes: ", probe.value_size())
print("value at first probe: ", probe.get_probe(0))



#!/usr/bin/env python
#
# Copyright (C) 2011 Marie E. Rognes
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Utility script for generating .rst documentation for DOLFIN

from __future__ import print_function
import os, sys

dolfin_import_error_msg = """
Unable to import the %s module
Error: %s
Did you forget to update your PYTHONPATH variable?"""

try:
    import dolfin_utils
except Exception as what:
    raise ImportError(dolfin_import_error_msg % ("dolfin_utils", what))

from dolfin_utils.documentation import generate_cpp_api_documentation
from dolfin_utils.documentation import generate_python_api_documentation

def generate_dolfin_doc(input_dir, output_dir, version=None):

    if version is None:
        version = "dev"

    # Make output directory (or use current if existing)
    try:
        os.makedirs(output_dir)
    except:
        pass

    # Generate .rst for C++ documentation
    api_output_dir = os.path.join(output_dir, "cpp", "programmers-reference")
    generate_cpp_api_documentation(input_dir, api_output_dir, version)

    # Try to import DOLFIN Python module
    module_name = "dolfin"
    try:
        exec("import %s" % module_name)
        exec("module = %s" % module_name)
    except Exception as what:
        raise ImportError(dolfin_import_error_msg % (module_name, what))

    # Generate .rst for Python documentation
    api_output_dir = os.path.join(output_dir, "python", "programmers-reference")
    generate_python_api_documentation(module, api_output_dir, version)

    print("\nSuccessfully generated API documentation.\n")

if __name__ == "__main__":

    args = sys.argv[1:]

    if len(args) != 3:
        usage= "Usage: python generate_api_doc.py source_dir output_dir version"
        print(usage)
        sys.exit(2)

    generate_dolfin_doc(args[0], args[1], args[2])



"""Unit tests for parameter library"""

# Copyright (C) 2011 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function
import pytest
import os

from dolfin import *
from dolfin_utils.test import *

@skip_in_parallel
def test_simple(tempdir):

    # Create some parameters
    p0 = Parameters("test")
    p0.add("filename", "foo.txt")
    p0.add("maxiter", 100)
    p0.add("tolerance", 0.001)
    p0.add("monitor_convergence", True)

    # Save to file
    f0 = File(os.path.join(tempdir, "test_parameters.xml"))
    f0 << p0

    # Read from file
    p1 = Parameters()
    f1 = File(os.path.join(tempdir, "test_parameters.xml"))
    f1 >> p1

    # Check values
    assert p1.name() == "test"
    assert p1["filename"] == "foo.txt"
    assert p1["maxiter"] == 100
    assert p1["tolerance"] == 0.001
    assert p1["monitor_convergence"] == True

@skip_in_parallel
def test_gzipped_simple(tempdir):

    # Create some parameters
    p0 = Parameters("test")
    p0.add("filename", "foo.txt")
    p0.add("maxiter", 100)
    p0.add("tolerance", 0.001)
    p0.add("monitor_convergence", True)

    # Save to file
    f0 = File(os.path.join(tempdir, "test_parameters.xml.gz"))
    f0 << p0

    # Read from file
    p1 = Parameters()
    f1 = File(os.path.join(tempdir, "test_parameters.xml.gz"))
    f1 >> p1

    # Check values
    assert p1.name() == "test"
    assert p1["filename"] == "foo.txt"
    assert p1["maxiter"] == 100
    assert p1["tolerance"] == 0.001
    assert p1["monitor_convergence"] == True

@skip_in_parallel
def test_nested(tempdir):

    # Create some nested parameters
    p0 = Parameters("test")
    p00 = Parameters("sub0")
    p00.add("filename", "foo.txt")
    p00.add("maxiter", 100)
    p00.add("tolerance", 0.001)
    p00.add("monitor_convergence", True)
    p0.add("foo", "bar")
    p01 = Parameters(p00);
    p01.rename("sub1");
    p0.add(p00)
    p0.add(p01)

    # Save to file
    f0 = File(os.path.join(tempdir, "test_parameters.xml"))
    f0 << p0

    # Read from file
    p1 = Parameters()
    f1 = File(os.path.join(tempdir, "test_parameters.xml"))
    f1 >> p1

    # Check values
    assert p1.name() == "test"
    assert p1["foo"] == "bar"
    assert p1["sub0"]["filename"] == "foo.txt"
    assert p1["sub0"]["maxiter"] == 100
    assert p1["sub0"]["tolerance"] == 0.001
    assert p1["sub0"]["monitor_convergence"] == True

@skip_in_parallel
def test_parameters_update(tempdir):
    p0 = parameters.copy()
    p1 = parameters.copy()
    p1.update(p0)

@skip_in_parallel
def test_nested_read_existing(tempdir):
    """Test that we can read in a nested parameter database into
       an existing (and matching) parameter database"""

    file = File(os.path.join(tempdir, "test_parameters.xml"))
    file << parameters

    p = Parameters("test")
    file >> p
    file >> p

@skip_in_parallel
def test_solver_parameters():
    "Test that global parameters are propagated to solvers"

    # Record default values so we can change back
    absolute_tolerance = parameters["krylov_solver"]["absolute_tolerance"]

    # Set global parameters
    parameters["krylov_solver"]["absolute_tolerance"] = 1.23456

    # Create solvers
    krylov_solver = KrylovSolver()

    # Check that parameters propagate to solvers
    assert krylov_solver.parameters["absolute_tolerance"] == 1.23456

    # Reset parameters so that other tests will continue to work
    if absolute_tolerance is not None:
        parameters["krylov_solver"]["absolute_tolerance"] = absolute_tolerance



# -*- coding: utf-8 -*-
"""This module provides the following constants:

* DOLFIN_EPS = 3.0e-16
* DOLFIN_EPS_LARGE = 1e-14
* DOLFIN_SQRT_EPS = 1e-8
* DOLFIN_PI = 3.141592653589793238462

* FENICS_EPS = 3.0e-16
* FENICS_EPS_LARGE = 1e-14
* FENICS_SQRT_EPS = 1e-8
* FENICS_PI = 3.141592653589793238462
"""

__all__ = ['DOLFIN_EPS', 'DOLFIN_EPS_LARGE', 'DOLFIN_SQRT_EPS', 'DOLFIN_PI',
           'FENICS_EPS', 'FENICS_EPS_LARGE', 'FENICS_SQRT_EPS', 'FENICS_PI']

from dolfin.cpp import DOLFIN_EPS, DOLFIN_EPS_LARGE, DOLFIN_SQRT_EPS, DOLFIN_PI
from dolfin.cpp import FENICS_EPS, FENICS_EPS_LARGE, FENICS_SQRT_EPS, FENICS_PI



# -*- coding: utf-8 -*-
"Utility functions for docstring manipulations."

# Copyright (C) 2010 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Kristian B. Oelgaard, 2010.
# Modified by Marie E. Rognes, 2011.
#
# First added:  2010-08-26
# Last changed: 2011-05-10

import re

_compiled_cache = {}

def indent(string, num_spaces):
    "Indent given text block given number of spaces"
    return "\n".join(num_spaces*" " + l for l in string.split("\n"))

def add_links(text, classnames, link_type):
    "Add crosslinks for classes"
    for classname in classnames:
        if classname not in text:
            continue
        p = _compiled_cache.get(classname)
        if not p:
            p = re.compile(r"\b_%s_\b" % classname)
            _compiled_cache[classname] = p
        text = p.sub("%s`%s`" % (link_type, classname), text)

    # Escape pointer arguments: replace * by \* after class names
    text = text.replace("`*", "`\*")

    return text




"""
Generate SWIG docstrings when doxygen is not installed
 
SWIG generates different Python modules depending on SWIG docstrings
being present. See dolfin issue 

 https://bitbucket.org/fenics-project/dolfin/issues/834/
    swig-generates-different-code-when-having
    
This script is based on dolfin's previous docstring generation script
and uses the dolfin_utils C++ parser and produces dummy docstrings.
It can be removed when SWIG no longer has this strange behaviour.

We currently (spring 2017) use a SWIG docstring generation process based
on running the doxygen C++ documentation tool. This gives us a more
standard documentation pipeline and we also get warnings if docstrings
fall out of sync with the function signatures.

This file is a heavily feature restricted version of the old implementation
of generate-swig-doctrings.py which can last be found in dolfin commit
7f40b0ecf687e3c56ce0fec7c7594d8ea2476784 and can be seen on bitbucket:

 https://bitbucket.org/fenics-project/dolfin/src/
    7f40b0ecf687e3c56ce0fec7c7594d8ea2476784/cmake/scripts/
    generate-swig-docstrings.py?fileviewer=file-view-default

This script assumes that all functions and classes lives in the dolfin namespace.
"""

# Copyright (C) 2010 Kristian B. Oelgaard
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Johan Hake 2010
# Modified by Anders E. Johansen 2011
# Modified by Anders Logg 2014
# Modified by Chris Richardson 2016
# Modified by Tormod Landet 2017

from __future__ import print_function
import os, shutil, types, sys

# Add path to dolfin_utils and import the documentation extractor.
doc_dir = os.path.abspath("site-packages")
sys.path.append(doc_dir)

from dolfin_utils.documentation import extract_doc_representation

docstring = '%%feature("docstring")  %s "\nDummy docstring. Reason: %s\n";\n\n'


def get_function_name(signature):
    "Extract function name from signature."
    words = signature.split("(")[0].split()
    # Special handling of operator since Swig needs 'operator double', not just
    # 'double', which is different from _normal_ operators like 'operator='
    if len(words) > 1 and words[-2] == "operator":
        return " ".join(words[-2:])
    return words[-1]


def group_overloaded_functions(docs):
    """Group functions with same name, but different signature.
    Assuming that overloaded functions in the dolfin namespace are defined
    in the same header file."""

    new_docs = []
    for (classname, parent, comment, function_documentation) in docs:
        func_doc = {}
        order = []
        # Iterate over class functions
        for (signature, comm) in function_documentation:
            # No need to put empty docstrings in the docstrings.i file!
            if comm is None:
                continue
            name = get_function_name(signature)
            if not name in order:
                order.append(name)
            if not name in func_doc:
                func_doc[name] = [(signature, comm)]
            else:
                func_doc[name].append((signature, comm))
        new_docs.append((classname, parent, comment, func_doc, order))

    return new_docs


def write_docstrings(output_file, module, header, docs, classnames, reason='Missing doxygen'):
    """Write docstrings from a header file."""

    output_file.write("// Documentation extracted from: (module=%s, header=%s)\n" % (module, header))

    documentation = group_overloaded_functions(docs)
    for (classname, parent, comment, func_docs, order) in documentation:
        # Create class documentation (if any) and write.
        if classname is not None and comment is not None:
            output_file.write(docstring % ("dolfin::%s" % classname, reason))
        # Handle functions in the correct order (according to definition in the
        # header file).
        for name in order:
            func_name = "dolfin::%s::%s" % (classname, name)
            if classname is None:
                func_name = "dolfin::%s" % name

            functions = func_docs[name]
            if not functions:
                continue
            output_file.write(docstring % (func_name, reason))


def generate_dummy_docstrings(top_destdir, reason='Missing doxygen'):
    """
    Generate docstring files for each module
    """
    # Get top DOLFIN directory.
    script_rel_path = os.sep.join(__file__.split(os.sep)[:-1])
    script_rel_path = script_rel_path or "."
    dolfin_dir = os.path.abspath(os.path.join(script_rel_path, os.pardir, os.pardir))
    top_destdir = top_destdir or dolfin_dir
    abs_destdir = top_destdir if os.path.isabs(top_destdir) else os.path.join(dolfin_dir, top_destdir)

    if not os.path.isdir(abs_destdir):
        raise RuntimeError("%s is not a directory." % abs_destdir)

    # Extract documentation
    documentation, classnames = extract_doc_representation(dolfin_dir)

    print("Generating dummy docstrings...")
    for module in documentation:
        if not os.path.isdir(os.path.join(abs_destdir, "dolfin", "swig", module)):
            os.mkdir(os.path.join(abs_destdir, "dolfin", "swig", module))
        outpath = os.path.join(abs_destdir, "dolfin", "swig", module, "docstrings.i")
        output_file = open(outpath, "wt")
        output_file.write("// Dummy docstrings file, extracted from the DOLFIN source C++ files.\n\n")

        print("  Writing ", outpath)
        for header, docs in documentation[module]:
            write_docstrings(output_file, module, header, docs, classnames, reason)

        output_file.close()


if __name__ == "__main__":
    dest_dir = sys.argv[1] if len(sys.argv) > 2 else ""
    generate_dummy_docstrings(dest_dir)



# -*- coding: utf-8 -*-
"""This module defines some special functions
(originally defined in SpecialFunctions.h)."""

# Copyright (C) 2008-2014 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Johan Hake 2008-2009
# Modified by Garth N. Wells 2010
# Modified by Martin Sandve Alnæs 2013-2014

__all__ = ["MeshCoordinates", "FacetArea", "FacetNormal", "CellSize", "CellVolume",
           'SpatialCoordinate', 'CellNormal', 'CellDiameter', 'Circumradius',
           'MinCellEdgeLength', 'MaxCellEdgeLength',
           'MinFacetEdgeLength', 'MaxFacetEdgeLength']

# Import UFL and SWIG-generated extension module (DOLFIN C++)
import ufl
import dolfin.cpp as cpp

# Local imports
from dolfin.functions.expression import Expression

def _mesh2domain(mesh):
    "Deprecation mechanism for symbolic geometry."

    # Handle MultiMesh
    if isinstance(mesh, cpp.MultiMesh):
        mesh = mesh.part(0)

    if isinstance(mesh, ufl.cell.AbstractCell):
        raise TypeError("Cannot construct geometry from a Cell. Pass the mesh instead, for example use FacetNormal(mesh) instead of FacetNormal(triangle) or triangle.n")
    return mesh.ufl_domain()

class MeshCoordinates(Expression, ufl.Coefficient, cpp.MeshCoordinates):
    def __init__(self, mesh):
        "Create function that evaluates to the mesh coordinates at each vertex."
        # Initialize C++ part
        cpp.MeshCoordinates.__init__(self, mesh)

        # Initialize UFL part
        ufl_element = mesh.ufl_domain().ufl_coordinate_element()
        if ufl_element.family() != "Lagrange" or ufl_element.degree() != 1:
            cpp.dolfin_error("specialfunctions.py",
                             "initialize MeshCoordinates",
                             "dolfin::MeshCoordinates only supports affine meshes")
        ufl_function_space = ufl.FunctionSpace(mesh.ufl_domain(), ufl_element)
        ufl.Coefficient.__init__(self, ufl_function_space, count=self.id())

MeshCoordinates.__doc__ = cpp.MeshCoordinates.__doc__

class FacetArea(Expression, ufl.Coefficient, cpp.FacetArea):
    def __init__(self, mesh):
        """
        Create function that evaluates to the facet area/length on each facet.

        *Arguments*
            mesh
                a :py:class:`Mesh <dolfin.cpp.Mesh>`.

        *Example of usage*

            .. code-block:: python

                mesh = UnitSquare(4,4)
                fa = FacetArea(mesh)

        """

        # Handle MultiMesh
        if isinstance(mesh, cpp.MultiMesh):
            mesh = mesh.part(0)

        # Initialize C++ part
        cpp.FacetArea.__init__(self, mesh)

        # Initialize UFL part
        # NB! This is defined as a piecewise constant function for each cell, not for each facet!
        ufl_element = ufl.FiniteElement("Discontinuous Lagrange", mesh.ufl_cell(), 0)
        ufl_function_space = ufl.FunctionSpace(mesh.ufl_domain(), ufl_element)
        ufl.Coefficient.__init__(self, ufl_function_space, count=self.id())

FacetArea.__doc__ = cpp.FacetArea.__doc__

# Simple definition of FacetNormal via UFL
def FacetNormal(mesh):
    """
    Return symbolic facet normal for given mesh.

    *Arguments*
        mesh
            a :py:class:`Mesh <dolfin.cpp.Mesh>`.


    *Example of usage*

        .. code-block:: python

            mesh = UnitSquare(4,4)
            n = FacetNormal(mesh)

    """

    return ufl.FacetNormal(_mesh2domain(mesh))

# Simple definition of CellSize via UFL
def CellDiameter(mesh):
    """
    Return function cell diameter for given mesh.

    Note that diameter of cell :math:`K` is defined as
    :math:`\sup_{\mathbf{x,y}\in K} |\mathbf{x-y}|`.

    *Arguments*
        mesh
            a :py:class:`Mesh <dolfin.cpp.Mesh>`.

    *Example of usage*

        .. code-block:: python

            mesh = UnitSquare(4,4)
            h = CellDiameter(mesh)

    """
    return ufl.CellDiameter(_mesh2domain(mesh))

# Simple definition of CellSize via UFL
def CellSize(mesh):
    """
    Return function twice the cell circumradius for given mesh.

    This does not generalize to quadrilateral/hexahedral cells
    and the function name is misleading. The function is hence
    deprecated. Use ``CellDiameter`` (or ``2*Circumradius`` if
    you really want that).

    *Arguments*
        mesh
            a :py:class:`Mesh <dolfin.cpp.Mesh>`.

    *Example of usage*

        .. code-block:: python

            mesh = UnitSquare(4,4)
            D = CellSize(mesh)

    """
    cpp.deprecation("\"CellSize\"",
                    "2017.2.0",
                    "Use \"CellDiameter\" or \"2*Circumradius\".")
    return 2.0*ufl.Circumradius(_mesh2domain(mesh))

# Simple definition of CellVolume via UFL
def CellVolume(mesh):
    """
    Return symbolic cell volume for given mesh.

    *Arguments*
        mesh
            a :py:class:`Mesh <dolfin.cpp.Mesh>`.

    *Example of usage*

        .. code-block:: python

            mesh = UnitSquare(4,4)
            vol = CellVolume(mesh)

    """

    return ufl.CellVolume(_mesh2domain(mesh))

# Simple definition of SpatialCoordinate via UFL
def SpatialCoordinate(mesh):
    """
    Return symbolic physical coordinates for given mesh.

    *Arguments*
        mesh
            a :py:class:`Mesh <dolfin.cpp.Mesh>`.

    *Example of usage*

        .. code-block:: python

            mesh = UnitSquare(4,4)
            x = SpatialCoordinate(mesh)

    """

    return ufl.SpatialCoordinate(_mesh2domain(mesh))

# Simple definition of CellNormal via UFL
def CellNormal(mesh):
    """
    Return symbolic cell normal for given manifold mesh.

    *Arguments*
        mesh
            a :py:class:`Mesh <dolfin.cpp.Mesh>`.

    *Example of usage*

        .. code-block:: python

            mesh = UnitSquare(4,4)
            n = CellNormal(mesh)

    """

    return ufl.CellNormal(_mesh2domain(mesh))

# Simple definition of Circumradius via UFL
def Circumradius(mesh):
    """
    Return symbolic cell circumradius for given mesh.

    *Arguments*
        mesh
            a :py:class:`Mesh <dolfin.cpp.Mesh>`.

    *Example of usage*

        .. code-block:: python

            mesh = UnitSquare(4,4)
            R = Circumradius(mesh)

    """

    return ufl.Circumradius(_mesh2domain(mesh))

# Simple definition of MinCellEdgeLength via UFL
def MinCellEdgeLength(mesh):
    """
    Return symbolic minimum cell edge length of a cell for given mesh.

    *Arguments*
        mesh
            a :py:class:`Mesh <dolfin.cpp.Mesh>`.

    *Example of usage*

        .. code-block:: python

            mesh = UnitSquare(4,4)
            mince = MinCellEdgeLength(mesh)

    """

    return ufl.MinCellEdgeLength(_mesh2domain(mesh))

# Simple definition of MaxCellEdgeLength via UFL
def MaxCellEdgeLength(mesh):
    """
    Return symbolic maximum cell edge length of a cell for given mesh.

    *Arguments*
        mesh
            a :py:class:`Mesh <dolfin.cpp.Mesh>`.

    *Example of usage*

        .. code-block:: python

            mesh = UnitSquare(4,4)
            maxce = MaxCellEdgeLength(mesh)

    """

    return ufl.MaxCellEdgeLength(_mesh2domain(mesh))

# Simple definition of MinFacetEdgeLength via UFL
def MinFacetEdgeLength(mesh):
    """
    Return symbolic minimum facet edge length of a cell for given mesh.

    *Arguments*
        mesh
            a :py:class:`Mesh <dolfin.cpp.Mesh>`.

    *Example of usage*

        .. code-block:: python

            mesh = UnitSquare(4,4)
            minfe = MinFacetEdgeLength(mesh)

    """

    return ufl.MinFacetEdgeLength(_mesh2domain(mesh))

# Simple definition of MaxFacetEdgeLength via UFL
def MaxFacetEdgeLength(mesh):
    """
    Return symbolic maximum facet edge length of a cell for given mesh.

    *Arguments*
        mesh
            a :py:class:`Mesh <dolfin.cpp.Mesh>`.

    *Example of usage*

        .. code-block:: python

            mesh = UnitSquare(4,4)
            maxfe = MaxFacetEdgeLength(mesh)

    """

    return ufl.MaxFacetEdgeLength(_mesh2domain(mesh))



# -*- coding: utf-8 -*-
"""Convenience functions that are useful in subdomains and expressions. Matches
some of the functions in dolfin/math/basic.h."""

__all__ = ["sqr"]

from . constants import DOLFIN_EPS

# The cpp version of these functions are about 3 times faster
#def between(x0, x, x1):
#    """Returns true if x0-epsilon < x < x1+epsilon"""
#    return x0-DOLFIN_EPS < x < x1+DOLFIN_EPS
#
#def near(x, x0):
#    """Returns true if x0-epsilon < x < x0+epsilon"""
#    return x0-DOLFIN_EPS < x < x0+DOLFIN_EPS

def sqr(x):
    return x*x



# -*- coding: utf-8 -*-
"""This module provides functionality to compile PyDOLFIN compatible
extension modules."""

# Copyright (C) 2009-2016 Johan Hake
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Johannes Ring, 2011
# Modified by Mikael Mortensen, 2014
# Modified by Martin Sandve Alnæs, 2016

import sys
import re
import hashlib

import instant
from ufl.utils.sorting import canonicalize_metadata
import ffc
from dolfin_utils.cppparser import *

# Import PyDOLFIN
import dolfin
import dolfin.cpp as cpp
from dolfin.cpp import parameters

from dolfin.compilemodules.jit import mpi_jit_decorator
from dolfin.compilemodules.swigimportinfo import *

__all__ = ["compile_extension_module",
           "expression_to_code_fragments",
           "math_header"]

# Bump the interface version if anything changes that invalidates
# cached modules (not required for change in generated code, swig
# version or dolfin version)
_interface_version = 8

# Add max_signature_length to parameter system (don't need this in C++)
cpp.parameters.add("max_signature_length", 0)

# A list of supported math builtins
_cpp_math_builtins = [
    # <cmath> functions: from http://www.cplusplus.com/reference/cmath/
    "cos", "sin", "tan", "acos", "asin", "atan", "atan2",
    "cosh", "sinh", "tanh", "exp", "frexp", "ldexp", "log", "log10", "modf",
    "pow", "sqrt", "ceil", "fabs", "floor", "fmod",
    "max", "min"]

_math_builtins = [
    # math.h functions: http://en.wikibooks.org/wiki/C_Programming/C_Reference/math.h
    "acos", "asin", "atan", "atan2", "ceil", "cos", "cosh", "exp",
    "fabs", "floor", "fmod", "frexp", "ldexp", "log", "log10", "modf",
    "pow", "sin", "sinh", "sqrt", "tan", "tanh", "acosh", "asinh", "atanh",
    "cbrt", "copysign", "erf", "erfc", "exp2", "expm1", "fdim", "fma", "fmax",
    "fmin", "hypot", "ilogb", "lgamma", "llrint", "lrint", "llround", "lround",
    "log1p", "log2", "logb", "nan", "nearbyint", "nextafter", "nexttoward",
    "remainder", "remquo", "rint", "round", "scalbln", "scalbn", "tgamma", "trunc"]

_math_dolfin = [
    # functions from dolfin::math:
    "sqr", "ipow", "rand", "near", "DOLFIN_EPS", "DOLFIN_PI", "pi"]

_all_math = (list(set(_math_builtins).difference(_cpp_math_builtins))
            + _math_dolfin + _cpp_math_builtins)

math_header = """
// cmath functions
%s

const double pi = DOLFIN_PI;
""" % "\n".join("using std::%s;" % mf for mf in _cpp_math_builtins)


# Copied from http://en.cppreference.com/w/cpp/keyword 2017/02/02
_allowed_cpp_keywords = """and
and_eq
bitand
bitor
bool
char
char16_t
char32_t
double
false
float
int
long
not
not_eq
or
or_eq
short
signed
sizeof
true
unsigned
wchar_t
xor
xor_eq
""".splitlines()

_disallowed_cpp_keywords = """alignas
alignof
asm
atomic_cancel
atomic_commit
atomic_noexcept
auto
break
case
catch
class
compl
concept
const
constexpr
const_cast
continue
decltype
default
delete
do
dynamic_cast
else
enum
explicit
export
extern
for
friend
goto
if
import
inline
module
mutable
namespace
new
noexcept
nullptr
operator
private
protected
public
register
reinterpret_cast
requires
return
static
static_assert
static_cast
struct
switch
synchronized
template
this
thread_local
throw
try
typedef
typeid
typename
union
using
virtual
void
volatile
while""".splitlines()


# User defined attributes with same names as attributes
# of any compiled expression base class cause confusion
# and segfaults with swig
_fenics_keywords = [
    # Attributes of ufl.Expr and ufl.Coefficient
    "T",
    "dx",
    "evaluate",
    "geometric_dimension",
    "count",
    # Attributes of dolfin Variable, GenericFunction, and Function
    "id",
    "eval",
    "value_rank",
    "value_dimension",
    "value_size",
    "restrict",
    "update",
    "vector",
    "in",
    "interpolate",
    "extrapolate",
    ]


_additional_declarations = r"""
%%init%%{
import_array();
%%}

// Include global SWIG interface files:
// Typemaps, shared_ptr declarations, exceptions, version
%%include <std_shared_ptr.i>

// Global typemaps and forward declarations
%%include "dolfin/swig/typemaps/includes.i"
%%include "dolfin/swig/forwarddeclarations.i"

// Global exceptions
%%include <exception.i>
%%include "dolfin/swig/exceptions.i"

// Do not expand default arguments in C++ by generating two an extra
// function in the SWIG layer. This reduces code bloat.
%%feature("compactdefaultargs");

// STL SWIG string class
%%include <std_string.i>

// Manually import ufc:
%%{
#include <ufc.h>
%%}
%%shared_ptr(ufc::function)
%%shared_ptr(ufc::dofmap)
%%shared_ptr(ufc::finite_element)
%%shared_ptr(ufc::coordinate_mapping)
%%shared_ptr(ufc::function)
%%shared_ptr(ufc::form)
%%ignore ufc::integral;
%%ignore ufc::cell_integral;
%%ignore ufc::exterior_facet_integral;
%%ignore ufc::interior_facet_integral;
%%ignore ufc::vertex_integral;
%%ignore ufc::custom_integral;
%%ignore ufc::cutcell_integral;
%%ignore ufc::interface_integral;
%%ignore ufc::overlap_integral;
%%include <ufc.h>

// Local shared_ptr declarations
%(shared_ptr_declarations)s

%(additional_declarations)s

// Import statements
%(dolfin_import_statement)s

%%feature("autodoc", "1");

%%inline %%{
int get_swigversion() { return  SWIGVERSION; }
%%}

%%pythoncode %%{
tmp = hex(get_swigversion())
swigversion = "%%d.%%d.%%d"%%(tuple(map(int, [tmp[-5], tmp[-3], tmp[-2:]])))
del tmp, get_swigversion
%%}

"""

re_symbol = re.compile(r"(?:[a-zA-Z_]*::)*[\w\.]+")
re_number = re.compile("[-+]?(\d+(\.\d*)?|\.\d+)([eE][-+]?\d+)?")


def _check_valid_symbol(sym):
    if sym in _disallowed_cpp_keywords:
        cpp.dolfin_error("compilemodule.py",
                         "parse expression string",
                         "Detected C++ keyword (%s) in C++ expression" % sym)
    if sym in _fenics_keywords:
        cpp.dolfin_error("compilemodule.py",
                         "parse expression string",
                         "Detected name reserved by FEniCS (%s) in C++ expression" % sym)
    if sym.startswith("_") or sym.endswith("_"):
        cpp.dolfin_error("compilemodule.py",
                         "parse expression string",
                         "Detected invalid name with trailing underscores (%s) in C++ expression" % sym)
    if sym.startswith("ufl_"):
        cpp.dolfin_error("compilemodule.py",
                         "parse expression string",
                         "Detected invalid name with initial ufl_ (%s) in C++ expression" % sym)


def expression_to_code_fragments(expr, arguments,
        generic_function_members=(), mesh_function_members=()):
    "A help function which extract a dict with code snippets from an expression"
    expr = list(expr)

    # Autodetect variables from function strings
    variables = set()

    for i, c in enumerate(expr):
        # Find groups of connected alphanumeric letters
        symbols = re_symbol.findall(c)
        assert isinstance(symbols, list)
        variables.update(symbols)
        for sym in symbols:
            _check_valid_symbol(sym)
        # FIXME: Remove outcommented code?
        # NOTE: A hack to get around an ambiguous overloading of
        #       dolfin::pow(double,int)

        #if "pow" in symbols:
        #    c = c.replace("pow", "std::pow")
        #expr[i] = c

    # Remove any variables defined in the arguments list
    variables.difference_update(arguments)

    # Remove the builtin math functions from the variables
    variables.difference_update(_all_math)

    # Remove allowed C++ keywords
    variables.difference_update(_allowed_cpp_keywords)

    # Remove the numerals from the variables
    numerals = [v for v in variables if re_number.match(v)]
    variables.difference_update(numerals)

    # Remove namespace-specified variables
    namespacevars = [v for v in variables if "::" in v]
    variables.difference_update(namespacevars)

    # Variables are either GenericFunction, MeshFunction, or scalar
    mesh_function_member_names = [item[0] for item in mesh_function_members]
    scalar_members = sorted(variables - set(generic_function_members) - set(mesh_function_member_names))

    # Generate code for member variables
    members_code = ["  double %s;" % name for name in scalar_members]
    members_code.extend("  std::shared_ptr<dolfin::MeshFunction<%s>> shared_%s;" % (typename, name)
                        for name, typename in mesh_function_members)
    members_code.extend("  std::shared_ptr<dolfin::GenericFunction> shared_%s;" % name
                        for name in generic_function_members)
    members_code = "\n".join(members_code)

    # Generate constructor code for initialization of scalar member variables
    constructor_code = "\n".join("    %s = 0;" % name for name in scalar_members)

    # Connect the code fragments using the function template code
    fragments = {}
    fragments["members"]        = members_code
    fragments["constructor"]    = constructor_code

    # Return the code fragments
    return fragments, variables


def expect_list_of(argtype, arg, argname):
    if arg is None:
        return []

    if isinstance(arg, (list, tuple)):
        if all(isinstance(s, argtype) for s in arg):
            return arg

    cpp.dolfin_error("compilemodule.py",
                     "ensure correct argument for compile_extension_module",
                     "Provide a 'tuple' or 'list' with '%s', for the "\
                     "'%s' argument" % (argtype.__name__, argname))


def expect_arg(argtype, arg, argname):
    # Check the type of the argument
    if isinstance(arg, argtype):
        return

    cpp.dolfin_error("compilemodule.py",
                     "ensure correct argument for compile_extension_module",
                     "Provide a '%s', for the '%s' argument" % \
                     (argtype.__name__, argname))


def check_swig_version(compiled_module):

    # Check swig version of compiled module
    if compiled_module and compiled_module.swigversion != cpp.__swigversion__:
        cpp.dolfin_error("compilemodule.py",
                         "compiling extension module",
                         "Incompatible swig versions detected. DOLFIN swig "\
                         "version is not the same as extension module swig "\
                         "version: '%s' != '%s' " % \
                         (cpp.__swigversion__, compiled_module.swigversion))


@mpi_jit_decorator
def compile_extension_module(code, module_name="",
                             additional_declarations="",
                             additional_system_headers=None,
                             mpi_comm=None,
                             **instant_kwargs):
    """
    Just In Time compile DOLFIN C++ code into a Python module.

    *Arguments*
        code
            C++ code which implements any function or C++ class. Any function
            or class available in the C++ DOLFIN namespace can be used and/or
            subclassed. All typemaps from the original Python interface are
            available, making it possible to interface with for example NumPy
            for Array<double/int> arguments. Source which is not wrapped in
            a dolfin namespace will be automatically wrapped.

        module_name
            Force a name of the module. If not set a name based on the hex
            representation of the code will be used.

        additional_declarations
            Additional SWIG declarations can be passed using this argument.

        additional_system_headers :
            System headers needed to compile the generated can be included
            using this argument. The headers are passed using a list of 'str'

        mpi_comm
            An mpi communicator used to determine what processors
            should compile the module. By default mpi_world_comm is
            used, but could also accept mpi_group communicators.

    *Returns*

        The JIT compiled extension module

    *Examples of usage*

        The following toy example shows how one can use compiled extension
        modules to access low level PETSc routines:

        .. code-block:: python

            from numpy import arange
            code = '''
            namespace dolfin {

              void PETSc_exp(std::shared_ptr<dolfin::PETScVector> vec)
              {
                Vec x = vec->vec();
                assert(x);
                VecExp(x);
              }
            }
            '''
            ext_module = compile_extension_module(code,
                         additional_system_headers=["petscvec.h"])
            comm = mpi_comm_world()
            vec = PETScVector(comm, 10)
            vec[:] = arange(10)
            print vec[-1]
            ext_module.PETSc_exp(vec)
            print vec[-1]

    """
    # Check the provided arguments
    expect_arg(str, code, "first")
    expect_arg(str, module_name, "module_name")
    expect_arg(str, additional_declarations, "additional_declarations")
    additional_system_headers = \
                expect_list_of(str, additional_system_headers, "additional_system_headers")

    # Check that the code does not use 'using namespace dolfin'
    if re.search("using\s+namespace\s+dolfin",code):
        cpp.dolfin_error("compilemodule.py",
                         "ensure correct argument to compile_extension_module",
                         "Do not use 'using namespace dolfin'. "\
                         "Include the code in namespace dolfin {...} instead")

    # Check if the code does not use namespace dolfin {...}
    if not re.search("namespace\s+dolfin\s*\{[\s\S]+\}", code):

        # Wrap and indet code in namespace dolfin
        codelines = ["namespace dolfin","{"]
        codelines += ["  " + line for line in code.split("\n")]
        codelines += ["}"]
        code = "\n".join(codelines)

    # Create unique module name for this application run
    if not module_name:
        module_signature = hashlib.sha1((repr(code) +
                                   dolfin.__version__ +
                                   str(_interface_version) +
                                   ffc.ufc_signature() +
                                   sys.version +
                                   additional_declarations +
                                   str(additional_system_headers) +
                                   repr(canonicalize_metadata(instant_kwargs))
                                   ).encode("utf-8")
                                ).hexdigest()
        max_signature_length = parameters["max_signature_length"]
        if max_signature_length:
            module_signature = module_signature[:max_signature_length]
        module_name = "dolfin_%s" % module_signature

    # Extract dolfin dependencies and class names
    used_types, declared_types = parse_and_extract_type_info(code)

    # Add any bases of the declared types to used_types
    for declared_type, bases in list(declared_types.items()):
        used_types.update(bases)

    # Filter out dolfin types and add derived and bases for each type
    used_dolfin_types = []
    for dolfin_type in dolfin_type_def:
        for used_type in used_types:
            if dolfin_type in used_type:

                # Add bases and derived types
                used_dolfin_types.extend(\
                    dolfin_type_def[dolfin_type]["bases"])

                # Add dolfin type
                used_dolfin_types.append(dolfin_type)

                break

    # Generate dependency info
    dependencies = {}
    for dolfin_type in used_dolfin_types:
        if dolfin_type_def[dolfin_type]["submodule"] not in dependencies:
            dependencies[dolfin_type_def[dolfin_type]["submodule"]] = []

        dependencies[dolfin_type_def[dolfin_type]["submodule"]].append(\
            dolfin_type_def[dolfin_type]["header"])

    # Need special treatment for template definitions in function/pre.i
    if "function" in dependencies:
        for dolfin_type in ["FunctionSpace", "Function"]:
            dependencies["function"].append(dolfin_type_def[dolfin_type]["header"])

    # Add dependency for constants and types
    if "common" not in dependencies:
        dependencies["common"] = ["dolfin/common/constants.h", "dolfin/common/types.h"]

    else:
        for f in ["dolfin/common/constants.h", "dolfin/common/types.h"]:
            if f not in dependencies["common"]:
                dependencies["common"].append(f)

    # Sort the dependencies
    dependencies = sort_submodule_dependencies(dependencies, submodule_info)

    import_lines, headers_includes, file_dependencies = \
                  build_swig_import_info(dependencies, submodule_info, "dolfin.cpp.")

    # Extract header info
    dolfin_system_headers = [header for header in file_dependencies
                             if not "pre.i" in header]

    # Check the handed import files
    interface_import_files = []

    # Check cache
    compiled_module = instant.import_module(module_name)

    if not compiled_module:
        # Configure instant before building
        sys.stdout.flush()
        dolfin.info("Calling DOLFIN just-in-time (JIT) compiler, this may take some time.")

        # Add various system headers
        instant_kwargs["system_headers"] = ["cmath", "iostream","complex",
                                            "stdexcept","numpy/arrayobject.h",
                                            "memory",
                                            "dolfin/common/types.h",
                                            "dolfin/math/basic.h",
                                        ] + instant_kwargs.get("system_headers", [])
        instant_kwargs["system_headers"] += dolfin_system_headers
        instant_kwargs["system_headers"] += additional_system_headers

        # Add cmake packages
        instant_kwargs["cmake_packages"]  = ["DOLFIN"] + \
                                            instant_kwargs.get("cmake_packages", [])
        instant_kwargs["signature"] = module_name

        # Add additional swig declarations
        dolfin_import_statement = "\n".join(import_lines)
        shared_ptr_declarations = extract_shared_ptr_declaration(
            declared_types, used_dolfin_types, shared_ptr_classes)
        additional_declarations = _additional_declarations % {
            "additional_declarations": additional_declarations,
            "shared_ptr_declarations": shared_ptr_declarations,
            "dolfin_import_statement": dolfin_import_statement,
            }

        # Compile extension module with instant
        compiled_module = instant.build_module(
            code=code,
            additional_declarations=additional_declarations,
            **instant_kwargs)

    sys.stdout.flush()

    # Check that the swig version of the compiled module is the same as
    # dolfin was compiled with
    check_swig_version(compiled_module)

    return compiled_module


def extract_shared_ptr_declaration(declared_types, used_dolfin_types, shared_ptr_classes):
    " Extract any declaration for shared_ptr"
    # Check if there are any classes that is derived from any of the
    # shared_ptr classes in PyDOLFIN and declare if any
    shared_ptr_format = "%%shared_ptr(dolfin::%s)"

    used_shared_ptr_types = []

    # Collect used types which should be shared_ptr declared
    for dolfin_type in used_dolfin_types:
        if dolfin_type in shared_ptr_classes:
            used_shared_ptr_types.append(dolfin_type)

    used_shared_ptr_types.extend(derived for derived, bases in list(declared_types.items())
                                 if any(base in shared_ptr_classes for base in bases))

    shared_ptr_declarations = "\n".join([shared_ptr_format % derived
                                         for derived in used_shared_ptr_types])
    return shared_ptr_declarations



"""
Produce docstrings for SWIG.

We run doxygen and parse the resulting XML to generate ReStucturedText
"""
from __future__ import print_function
from codesnippets import copyright_statement
import os
import sys
import subprocess
from dummy_docstrings import generate_dummy_docstrings


PROBLEMATIC_DOXYGEN_VERSIONS = ['1.8.13']


def get_doxygen_version():
    """
    Test for presence of doxygen
    
    Returns None if doxygen is not present, else a string containing
    the doxygen version
    """
    try:
        ver = subprocess.check_output(['doxygen', '--version'])
    except Exception:
        return None
    return ver.decode('utf8', 'replace').strip()


def get_required_swig_files(dolfin_dir, swig_dir):
    """
    Return the names of the docstring.i files we are required to generate
    """
    swig_files = []
    dolfin_dir = os.path.join(dolfin_dir, 'dolfin')
    for subdir in os.listdir(dolfin_dir):
        path = os.path.join(dolfin_dir, subdir)
        if not os.path.isdir(path) or subdir == 'swig':
            continue
        swig_files.append(os.path.join(swig_dir, subdir, 'docstrings.i'))
    return swig_files


def generate_docstrings(top_destdir):
    """
    Generate docstring files for each module
    """
    doxyver = get_doxygen_version()
    
    if doxyver is None:
        print('--------------------------------------------')
        print('WARNING: Missing doxygen, producing dummy docstrings')
        generate_dummy_docstrings(top_destdir, reason='Missing doxygen')
        return
    
    # Get top DOLFIN directory.
    script_dir = os.path.dirname(os.path.abspath(__file__))
    dolfin_dir = os.path.abspath(os.path.join(script_dir, os.pardir,
                                              os.pardir))
    
    print('--------------------------------------------')
    print('Running doxygen to read docstrings from C++:')
    print('Doxygen version:', doxyver)
    
    dummy_reason = ''
    if str(doxyver) in PROBLEMATIC_DOXYGEN_VERSIONS:
        print('WARNING: this doxygen version has known problems with dolfin')
        dummy_reason = 'Buggy doxygen version %s. ' % doxyver
    
    sys.stdout.flush() # doxygen writes to stderr and mangles output order
    
    # Get doc directory and run doxygen there
    doc_dir = os.path.join(dolfin_dir, 'doc')
    xml_dir = os.path.join(dolfin_dir, 'doc', 'doxygen', 'xml')
    try:
        subprocess.call(['doxygen'], cwd=doc_dir)
    except OSError as e:
        print('ERROR: could not run doxygen:', e)
    print('DONE parsing C++ with doxygen')

    top_destdir = top_destdir or dolfin_dir
    abs_destdir = top_destdir if os.path.isabs(top_destdir) else os.path.join(dolfin_dir, top_destdir)

    if not os.path.isdir(abs_destdir):
        raise RuntimeError("%s is not a directory." % abs_destdir)

    # Directory with swig files
    swig_dir = os.path.join(abs_destdir, "dolfin", "swig")

    # Get copyright form
    copyright_form_swig = dict(comment=r"//", holder="Kristian B. Oelgaard, Tormod Landet")
    copyright_info = copyright_statement % copyright_form_swig

    # Get the doxygen parser and docstring formater
    sys.path.insert(0, doc_dir)
    from generate_api_rst import parse_doxygen_xml_and_generate_rst_and_swig
    
    # Remove destination files so that we can check if we succeeded
    required_swig_files = get_required_swig_files(dolfin_dir, swig_dir)
    for file_name in required_swig_files:
        if os.path.isfile(file_name):
            os.remove(file_name)
    
    print('--------------------------------------------')
    print('Generating python docstrings in directory')
    print('  swig_dir = %s' % swig_dir)
    
    # Extract documentation and generate docstrings
    if os.path.isdir(xml_dir):
        parse_doxygen_xml_and_generate_rst_and_swig(xml_dir=xml_dir,
                                                    api_gen_dir=None,
                                                    swig_dir=swig_dir,
                                                    swig_file_name="docstrings.i",
                                                    swig_header=copyright_info)
    else:
        # Doxygen crashed before generating any XML
        dummy_reason += 'Doxygen did not generate any XML files. '
    
    # Check that all files have been generated
    # Doxygen may have crashed halfway through generating the XML
    all_files_generated = True
    for file_name in required_swig_files:
        if not os.path.isfile(file_name):
            all_files_generated = False
            print('ERROR: missing file %s' % file_name)
    
    # Generate dummy files if something went wrong
    # Possible errors can be problems with the XML output from doxygen. As 
    # an example doxygen version 1.8.13 segfaults after producing a few XML
    # files while versions 1.8.12 and 1.8.14 works fine
    if not all_files_generated:
        print('--------------------------------------------')
        print('WARNING: Something went wrong, producing dummy docstrings')
        dummy_reason += 'Missing generated files. Did doxygen crash?'
        generate_dummy_docstrings(top_destdir, reason=dummy_reason)
    
    print('DONE generating docstrings')
    print('--------------------------------------------')


if __name__ == "__main__":
    if len(sys.argv) not in (1, 2):
        raise RuntimeError("expected 0 or 1 arguments")
    dest_dir = sys.argv[1] if len(sys.argv) > 2 else ""
    generate_docstrings(dest_dir)



"""Run Valgrind on all demos."""

# Copyright (C) 2008-2014 Ilmar Wilbers
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Anders Logg 2008
# Modified by Dag Lindbo 2008
# Modified by Johannes Ring 2008
# Modified by Johan Hake 2009

from __future__ import print_function
import sys, os, re
import platform
from instant import get_status_output

def run_and_analyse(path, run_str, prog_type, no_reachable_check = False):
    cmd = "%s %s" % (vg_comm, run_str)

    curdir = os.getcwd()
    os.chdir(path)
    try:
        status, output = get_status_output(cmd)
    finally:
        os.chdir(curdir)

    if "No such file or directory" in "".join([str(l) for l in output]):
        print("*** FAILED: Unable to run demo")
        return [(demo, "C++", output[1])]

    if len(re.findall('All heap blocks were freed',output[1])) == 1:
        print("OK")
        return []

    if "LEAK SUMMARY:" in output[1] and "ERROR SUMMARY:" in output[1]:
        if re_def_lost.search(output[1]) and \
           re_pos_lost.search(output[1]) and \
           re_error.search(output[1]) and \
           (no_reachable_check or re_reachable.search(output[1])):
            print("OK")
            return []

    print("*** FAILED: Memory error")
    return [(path, prog_type, output[1])]

def main():
    if "--only-python" in sys.argv:
        print("Skipping C++ only memory tests")
        sys.exit()

    if platform.system() in ['Darwin', 'Windows']:
        print("No support for Valgrind on this platform.")
        sys.exit(0)

    # Demos to run
    cppdemos = []
    for dpath, dnames, fnames in os.walk(os.path.join(os.curdir, "..", "..", "demo")):
        if os.path.basename(dpath) == 'cpp':
            if os.path.isfile(os.path.join(dpath, 'Makefile')):
                cppdemos.append(dpath)

    unit_test_excludes = ['graph']

    # Python unit test to run
    pythontests = []
    for dpath, dnames, fnames in os.walk(os.path.join(os.curdir, "..", "unit")):
        if os.path.basename(dpath) == 'python':
            if os.path.isfile(os.path.join(dpath, 'test.py')):
                pythontests.append(dpath)

    pythontests = [test for test in pythontests if not any([exclude in test for exclude in unit_test_excludes])]

    unit_test_excludes.append('meshconvert')

    # cpp unit test to run
    cpptests = []
    for dpath, dnames, fnames in os.walk(os.path.join(os.curdir, "..", "unit")):
        if os.path.basename(dpath) == 'cpp':
            if os.path.isfile(os.path.join(dpath, 'test')):
                cpptests.append(dpath)

    cpptests = [test for test in cpptests if not any([exclude in test for exclude in unit_test_excludes])]

    # Set non-interactive
    os.putenv('DOLFIN_NOPLOT', '1')

    # Helpful env vars
    os.putenv('G_SLICE','always-malloc')
    os.putenv('GLIBCXX_FORCE_NEW','1')
    os.putenv('G_DEBUG','gc-friendly')

    print(pythontests)

    # Demos that need command line arguments are treated seperately
    #cppdemos.remove('./../../demo/undocumented/quadrature/cpp')

    # Demos that are too time consuming to Valgrind
    cppdemos.remove('./../../demo/documented/cahn-hilliard/cpp')
    cppdemos.remove('./../../demo/undocumented/elastodynamics/cpp')

    re_def_lost = re.compile("definitely lost: 0 bytes in 0 blocks.")
    re_pos_lost = re.compile("possibly lost: 0 bytes in 0 blocks.")
    re_reachable = re.compile("still reachable: 0 bytes in 0 blocks.")
    re_error = re.compile("0 errors from 0 contexts")

    dolfin_supp = os.path.join(os.path.abspath(os.getcwd()), 'dolfin_valgrind.supp')
    vg_comm = 'valgrind --error-exitcode=9 --tool=memcheck --leak-check=full --show-reachable=yes --suppressions=%s' % dolfin_supp

    failed = []

    # Run C++ unittests
    print("----------------------------------------------------------------------")
    print("Running Valgrind on all C++ unittests")
    print("")
    print("Found %d C++ unittests" % len(cpptests))
    print("")

    for test_path in cpptests:
        print("----------------------------------------------------------------------")
        print("Running Valgrind on C++ unittest %s" % test_path)
        print("")
        if os.path.isfile(os.path.join(test_path, 'test')):
            failed += run_and_analyse(test_path,"./test","C++")
        else:
            print("*** Warning: missing test")

    # Outcommenting the Python unittests due to troubles with valgrind suppresions
    #
    #print "----------------------------------------------------------------------"
    #print "Running Valgrind on all Python unittests"
    #print ""
    #print "Found %d Python unittests" % len(pythontests)
    #print ""

    # Run Python unittests
    #for test_path in pythontests:
    #    print "----------------------------------------------------------------------"
    #    print "Running Valgrind on Python unittest %s" % test_path
    #    print ""
    #    if os.path.isfile(os.path.join(test_path, 'test.py')):
    #        failed += run_and_analyse(test_path,"python test.py","Python",True)
    #    else:
    #        print "*** Warning: missing test"

    # Run C++ demos
    print("----------------------------------------------------------------------")
    print("Running Valgrind on all demos (non-interactively)")
    print("")
    print("Found %d C++ demos" % len(cppdemos))
    print("")

    for demo_path in cppdemos:
        print("----------------------------------------------------------------------")
        print("Running Valgrind on C++ demo %s" % demo_path)
        print("")
        demo_name = "./demo_" + demo_path.split("/")[-2]
        print(demo_name)
        if os.path.isfile(os.path.join(demo_path, demo_name)):
            failed += run_and_analyse(demo_path, demo_name, "C++")
        else:
            print("*** Warning: missing demo")

    # Print output for failed tests
    print("")
    if len(failed) > 0:
        print("%d demo(s) and/or unit test(s) failed memcheck, see memcheck.log for details." % len(failed))
        file = open("memcheck.log", "w")
        for (test, interface, output) in failed:
            file.write("----------------------------------------------------------------------\n")
            file.write("%s (%s)\n" % (test, interface))
            file.write("\n")
            file.write(output)
            file.write("\n")
            file.write("\n")
    else:
        print("All demos and unit tests checked for memory leaks and errors: OK")

    # Return error code if tests failed
    return len(failed)

if __name__ == "__main__":
    sys.exit(main())



# -*- coding: utf-8 -*-
# Copyright (C) 2009 Anders Logg
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
#
# Modified by Johannes Ring, 2009.
#
# First added:  2009-09-16
# Last changed: 2009-09-16

from subprocess import Popen, PIPE, STDOUT

# These are taken from http://ivory.idyll.org/blog/mar-07/replacing-commands-with-subprocess

__all__ = ['getoutput', 'getstatusoutput']

def getoutput(cmd):
    "Replacement for commands.getoutput which does not work on Windows."
    pipe = Popen(cmd, shell=True, stdout=PIPE, stderr=STDOUT, bufsize=-1)
    r = pipe.wait()
    output, error = pipe.communicate()
    return output

def getstatusoutput(cmd, input=None, cwd=None, env=None):
    "Replacement for commands.getstatusoutput which does not work on Windows."
    pipe = Popen(cmd, shell=True, cwd=cwd, env=env, stdout=PIPE, stderr=STDOUT)
    output, error = pipe.communicate(input=input)
    assert not error
    status = pipe.returncode
    return status, output



#!/usr/bin/env python
#
# Read doxygen xml files into an object tree that can be
# written to either Sphinx ReStructuredText format or SWIG doctrings
#
# Written by Tormod Landet, 2017
#
from __future__ import print_function
import sys, os


try:
    import lxml.etree as ET
except ImportError:
    import xml.etree.ElementTree as ET


MOCK_REPLACEMENTS = {'operator[]': '__getitem__',
                     'operator()': '__call__',
                     'in': '_in',
                     'self': '_self',
                     'string': 'str',
                     'false': 'False',
                     'true': 'True',
                     'null': 'None'}


class Namespace(object):
    """
    A C++ namespace
    """
    def __init__(self, name):
        self.name = name # We use '' for the global namespace
        self.parent_namespace = None
        self.subspaces = {}
        self.members = {}

    def add(self, item):
        """
        Add a member to this namespace
        """
        if isinstance(item, Namespace):
            self.subspaces[item.name] = item
            item.parent_namespace = self
        else:
            self.members[item.name] = item
            item.namespace = self

    def lookup(self, name):
        """
        Find a named member. This is currently only used to find classes
        in namespaces. We use this to find the superclasses of each class
        since we only know their names and we need the list their
        superclasses as well to show only direct superclasses when we
        generate documentation for each class
        """
        item = self.members.get(name, None)
        if item: return item

        nn = name.split('::')[0]

        if nn == self.name:
            return None

        if nn in self.subspaces:
            return self.subspaces[nn].lookup(item)

        if self.parent_namespace is not None:
            return self.parent_namespace.lookup(name)


class Parameter(object):
    """
    A parameter to a C++ function
    """
    def __init__(self, name, param_type, value=None, description=''):
        self.name = name
        self.type = param_type
        self.value = None
        self.description = description

    @staticmethod
    def from_param(param_elem):
        """
        Read a <param> element and get name and type
        """
        tp = get_single_element(param_elem, 'type')
        param_type = description_to_string(tp, skipelems=('ref',))

        n1 = get_single_element(param_elem, 'declname', True)
        n2 = get_single_element(param_elem, 'defname', True)
        if n1 is None and n2 is not None:
            name = n2.text
        elif n1 is None:
            name = ''
        else:
            name = n1.text

        defval = None
        dv = get_single_element(param_elem, 'defval', True)
        if dv is not None:
            defval = dv.text

        description = ''
        doc1 = get_single_element(param_elem, 'briefdescription', True)
        if doc1 is not None:
            description = description_to_string(doc1)

        return Parameter(name, param_type, defval, description)


class NamespaceMember(object):
    """
    A class, function, enum, struct, or typedef
    """

    def __init__(self, name, kind):
        # Used for every member
        self.name = name
        self.kind = kind  # enum, struct, function or class
        self.namespace = None
        self.docstring = []
        self.short_name = None
        self.protection = None
        self.hpp_file_name = None
        self.xml_file_name = None

        # Used for functions and variables
        self.type = None
        self.type_description = ''

        # Used for functions
        self.parameters = []

        # Used for classes
        self.superclasses = []
        self.members = {}

        # Used for enums
        self.enum_values = []

    def add(self, item):
        """
        Add a member to a class
        """
        assert self.kind == 'class'
        self.members[item.name] = item
        item.namespace = self

    @staticmethod
    def from_memberdef(mdef, name, kind, xml_file_name, namespace_obj):
        """
        Read a memberdef element from a nameclass file (members of
        the namespace that are not classes) or from a compoundef
        (members of the class)
        """
        # Make sure we have the required "namespace::" prefix
        required_prefix = '%s::' % namespace_obj.name
        if kind != 'friend' and not name.startswith(required_prefix):
            name = required_prefix + name
        short_name = name[len(required_prefix):]

        if kind == 'function':
            argsstring = get_single_element(mdef, 'argsstring').text
            name += argsstring

        # Define attributes common to all namespace members
        item = NamespaceMember(name, kind)
        item.short_name = short_name
        item.protection = mdef.attrib['prot']
        item.xml_file_name = xml_file_name
        item.hpp_file_name = get_single_element(mdef, 'location').attrib['file']
        item._add_doc(mdef)

        # Get parameters (for functions)
        for param in mdef.findall('param'):
            item.parameters.append(Parameter.from_param(param))

        # Get type (return type for functions)
        mtype = get_single_element(mdef, 'type', True)
        if mtype is not None:
            item.type = description_to_string(mtype, skipelems=('ref',))

        # Get parameter descriptions
        dd = get_single_element(mdef, 'detaileddescription')
        for pi in findall_recursive(dd, 'parameteritem'):
            pnl = get_single_element(pi, 'parameternamelist')
            pd = get_single_element(pi, 'parameterdescription')
            param_desc = description_to_string(pd)

            pns = pnl.findall('parametername')
            for pn in pns:
                pname = pn.text
                pdesc = param_desc
                if 'direction' in pn.attrib:
                    pdesc += ' [direction=%s]' % pn.attrib['direction']
                maching_params = [p for p in item.parameters if p.name == pname]
                assert len(maching_params) == 1
                maching_params[0].description += ' ' + pdesc

        # Get return type description
        for ss in findall_recursive(dd, 'simplesect'):
            memory = {'skip_simplesect': False}
            if ss.get('kind', '') == 'return':
                item.type_description = description_to_string(ss, memory=memory)

        # Get enum values
        for ev in mdef.findall('enumvalue'):
            ename = get_single_element(ev, 'name').text
            evalue = ''
            init = get_single_element(ev, 'initializer', True)
            if init is not None:
                evalue = init.text
            item.enum_values.append((ename, evalue))

        return item

    @staticmethod
    def from_compounddef(cdef, name, kind, xml_file_name):
        """
        Read a compounddef element from a class definition file
        """
        item = NamespaceMember(name, kind)
        item.hpp_file_name = get_single_element(cdef, 'location').attrib['file']
        item.xml_file_name = xml_file_name
        item.short_name = name.split('::')[-1]
        item._add_doc(cdef)

        # Get superclasses with public inheritance
        igs = cdef.findall('collaborationgraph')
        if len(igs) == 1:
            for node in igs[0]:
                public = False
                for cn in node.findall('childnode'):
                    if cn.attrib.get('relation', '') == 'public-inheritance':
                        public = True
                if public:
                    label = get_single_element(node, 'label').text
                    if label != item.name:
                        item.superclasses.append(label)
        else:
            assert len(igs) == 0

        # Read members
        for s in cdef.findall('sectiondef'):
            members = s.findall('memberdef')
            for m in members:
                mname = get_single_element(m, 'name').text
                mkind = m.attrib['kind']
                mitem = NamespaceMember.from_memberdef(m, mname, mkind, xml_file_name, item)
                item.add(mitem)

        return item

    def _add_doc(self, elem):
        """
        Add docstring for the given element
        """
        bd = get_single_element(elem, 'briefdescription')
        dd = get_single_element(elem, 'detaileddescription')

        description_to_rst(bd, self.docstring)
        if self.docstring and self.docstring[-1].strip():
            self.docstring.append('')
        description_to_rst(dd, self.docstring)

    def get_superclasses(self):
        to_remove = set()
        for sn in self.superclasses:
            obj = self.namespace.lookup(sn)
            if obj:
                for sn2 in obj.superclasses:
                    if sn2 in self.superclasses:
                        to_remove.add(sn2)
        return [sn for sn in self.superclasses if sn not in to_remove]

    def _to_rst_string(self, indent, for_swig=False, for_mock=False):
        """
        Create a list of lines on Sphinx (ReStructuredText) format
        The header is included for Sphinx, but not for SWIG docstrings
        """
        ret = []
        if for_mock:
            for_swig = True

        if not for_swig:
            ret.append('')
            simple_kinds = {'typedef': 'type', 'enum': 'enum'}
            kinds_with_types = {'function': 'function', 'variable': 'var'}
            if self.kind in ('class', 'struct'):
                superclasses = self.get_superclasses()
                supers = ''
                if superclasses:
                    supers = ' : public ' + ', '.join(superclasses)
                ret.append(indent + '.. cpp:class:: %s%s' % (self.name, supers))
            elif self.kind in kinds_with_types:
                rst_kind = kinds_with_types[self.kind]
                ret.append(indent + '.. cpp:%s:: %s %s' % (rst_kind, self.type, self.name))
            elif self.kind in simple_kinds:
                rst_kind = simple_kinds[self.kind]
                ret.append(indent + '.. cpp:%s:: %s' % (rst_kind, self.name))
            else:
                raise NotImplementedError('Kind %s not implemented' % self.kind)

            # Docstring and parameters must be further indented
            indent += '   '
            ret.append(indent)

        # All: add docstring
        doclines = self.docstring
        if doclines and not doclines[0].strip():
            doclines = doclines[1:]
        ret.extend(indent + line for line in doclines)

        # Classes: separate friends from other members
        friends, members = [], []
        for _, member in sorted(self.members.items()):
            if member.kind == 'friend':
                friends.append(member)
            else:
                members.append(member)

        # Classes: add friends
        if friends:
            if ret and ret[-1].strip():
                ret.append(indent)
            ret.append(indent + 'Friends: %s.' % ', '.join(
                ':cpp:any:`%s`' % friend.name for friend in friends))

        # Functions: add space before parameters and return types
        if self.parameters or (self.type and for_swig) or self.type_description:
            if ret and ret[-1].strip():
                ret.append(indent)

        # Functions: add parameter definitions
        for param in self.parameters:
            if param.name:
                ptype = param.type.replace(':', '\\:')
                pname = param.name.replace(':', '\\:')
                if for_swig:
                    ret.append(indent + ':param %s %s: %s' % (ptype, pname, param.description))
                else:
                    # Parameter type is redundant info in the Sphinx produced html
                    ret.append(indent + ':param %s: %s' % (pname, param.description))

        # Functions: add return type (redundant info in the Sphinx produced html)
        if self.type and for_swig:
            ret.append(indent + ':rtype: %s' % self.type)
        if self.type_description:
            ret.append(indent + ':returns: %s' % self.type_description)

        # Enums: add enumerators
        for ename, evalue in self.enum_values:
            if ret and ret[-1].strip():
                ret.append(indent)
            ret.append(indent + '.. cpp:enumerator:: %s::%s %s' % (self.name, ename, evalue))
            ret.append(indent)

        # Remove doubled up blank lines
        if ret:
            ret2 = [ret[0]]
            for line in ret[1:]:
                if line.strip() or ret2[-1].strip():
                    ret2.append(line)
            ret = ret2

        # All: SWIG items are not nested, so we end this one here
        if for_swig and not for_mock:
            escaped = [line.replace('\\', '\\\\').replace('"', '\\"') for line in ret]
            sname = self.name
            if sname.endswith('=0'):
                sname = sname[:-2]
            elif sname.endswith('=delete'):
                sname = sname[:-7]
            ret = ['%%feature("docstring") %s "' % sname,
                   '\n'.join(escaped).rstrip() + '\n";\n']
            indent = ''

        # Classes: add members of a class
        if not for_mock:
            for member in members:
                ret.append(member._to_rst_string(indent, for_swig))

        return '\n'.join(ret)

    def to_swig(self):
        """
        Output SWIG docstring definitions
        """
        swigstr = self._to_rst_string(indent='', for_swig=True)
        # Get rid of repeated newlines
        for _ in range(4):
            swigstr = swigstr.replace('\n\n\n', '\n\n')

        # Remove any '=default' keywords
        swigstr = swigstr.replace("=default", "")

        return swigstr

    def to_rst(self, indent=''):
        """
        Output Sphinx :cpp:*:: definitions
        """
        return self._to_rst_string(indent)

    def to_mock(self, modulename, indent='', _classname=None):
        """
        Output mock Python code
        """
        # Get swig docstring and get rid of any repeated newlines
        new_indent = indent + '    '
        swigdoc = self._to_rst_string(indent=new_indent, for_mock=True)
        for _ in range(4):
            swigdoc = swigdoc.replace('\n\n\n', '\n\n')

        ret = []
        if self.kind == 'class':
            superclasses = [sc.split('::')[-1].split('<')[0] for sc in self.get_superclasses()]
            superclasses = [MOCK_REPLACEMENTS.get(sc, sc) for sc in superclasses]
            superclasses = [sc for sc in superclasses if sc != self.short_name]
            ret.append(indent + 'class %s:\n' % (self.short_name))
            ret.append(new_indent + '"""\n')
            ret.append(swigdoc.rstrip())
            ret.append('\n')
            for sc in superclasses:
                ret.append(new_indent + 'Superclass: :py:obj:`%s`\n' % sc)
            ret.append(new_indent + '"""\n')
            for member in self.members.values():
                ret.append('\n')
                ret.append(member.to_mock(modulename, indent=new_indent, _classname=self.short_name))

        elif self.kind == 'function':
            fname = MOCK_REPLACEMENTS.get(self.short_name, self.short_name)
            if fname.startswith('operator'):
                return ''
            params = [MOCK_REPLACEMENTS.get(p.name, p.name) for p in self.parameters]
            if _classname:
                params = ['self'] + params
                if _classname == fname:
                    fname = '__init__'
                elif '~' + _classname == fname:
                    fname = '__del__'

            if self.parameters and self.parameters[-1].type == '...':
                params[-1] = '*args'
            args = ', '.join(params)

            ret.append(indent + 'def %s(%s):\n' % (fname, args))
            ret.append(new_indent + '"""\n')
            ret.append(swigdoc.rstrip())
            ret.append('\n')
            ret.append(new_indent + '"""\n')
            ret.append(new_indent + 'print(WARNING)\n')

        elif self.kind == 'friend':
            pass

        elif self.kind == 'enum':
            ret.append(indent + '# Enumeration %s\n' % self.short_name)
            for ename, evalue in self.enum_values:
                ret.append(indent + '#: Enum value %s::%s %s\n' % (self.name, ename, evalue))
                if not evalue.strip():
                    evalue = '= None'
                evalue = MOCK_REPLACEMENTS.get(evalue[1:].strip(), evalue[1:].strip())
                ret.append(indent + '%s.%s_%s = %s\n' % (modulename, self.short_name, ename, evalue))

        elif self.kind == 'variable':
            pass
            #for line in swigdoc.split('\n'):
            #    ret.append(indent + '#: %s\n' % line)
            #ret.append(indent + 'module.%s = None\n' % self.short_name)

        elif self.kind == 'typedef':
            pass

        else:
            print('WARNING: kind %s not supported by to_mock()' % self.kind)

        if not _classname and self.kind in ('class', 'function'):
            ret.append('%s.%s = %s\n' % (modulename, self.short_name, self.short_name))

        return ''.join(ret)

    def __str__(self):
        return self.to_rst()


def description_to_string(element, indent='', skipelems=(), memory=None):
    lines = []
    description_to_rst(element, lines, indent, skipelems, memory)
    return ' '.join(lines).strip()


NOT_IMPLEMENTED_ELEMENTS = set()
def description_to_rst(element, lines, indent='', skipelems=(), memory=None):
    """
    Create a valid ReStructuredText block for Sphinx from the given description
    element. Handles <para> etc markup inside and also is called for every
    sub-tag of the description tag (like <para>, <ref> etc)
    """
    if lines == []:
        lines.append('')
    if memory is None:
        memory = dict()

    tag = element.tag
    children = list(element)
    postfix = ''
    postfix_lines = []

    # Handle known tags that show up in description type element trees
    # Tag contents are handled beneath, if the if-branch does not return
    # Unknown tags are just output unchanged and a WARNING is shown (in main)
    if tag in ('briefdescription', 'detaileddescription', 'parameterdescription',
               'type', 'highlight'):
        pass

    elif element in memory:
        # This element is being re-read, only treat the contained elements
        pass

    elif tag == 'para':
        if lines[-1].strip():
            lines.append(indent)
        postfix_lines.append(indent)

    elif tag == 'codeline':
        memory = dict(memory); memory[element] = 1
        skipelems = set(skipelems); skipelems.add('ref')
        line = description_to_string(element, indent, skipelems, memory)
        if line.startswith('*'):
            line = line[1:]
        lines.append(indent + line)
        return

    elif tag == 'ndash':
        lines[-1] += '--'

    elif tag == 'mdash':
        lines[-1] += '---'

    elif tag == 'sp':
        lines[-1] += ' '

    elif tag == 'ref':
        if 'ref' in skipelems:
            lines[-1] += element.text
        else:
            lines[-1] += ':cpp:any:`%s` ' % element.text
        return

    elif tag == 'ulink':
        lines[-1] += '`%s <%s>`_ ' % (element.text, element.get('url'))
        return

    elif tag == 'emphasis':
        if children and children[0].tag != 'ref':
            lines[-1] += '**'
            postfix += '**'

    elif tag == 'computeroutput':
        if element.text is None and not list(element):
            return
        elif lines[-1].endswith(':math:'):
            lines[-1] += '`'
            postfix += '` '
        else:
            lines[-1] += '``'
            postfix += '`` '

    elif tag in ('verbatim', 'programlisting'):
        if element.text is None and not list(element):
            return
        if lines[-1].strip():
            lines.append(indent)
        lines.append(indent + '::')
        postfix_lines.append(indent)
        indent += '   '
        lines.append(indent)
        lines.append(indent)

    elif tag == 'table':
        lines.extend(xml_table_to_rst(element, indent))
        lines.append(indent)
        return # Do not process children

    elif tag == 'formula':
        contents = element.text.strip()
        if contents.startswith(r'\['):
            if lines[-1].strip():
                lines.append(indent)
            lines.append(indent + '.. math::')
            lines.append(indent + '   ')
            lines.append(indent + '   ' + contents[2:-2])
            lines.append(indent)
            lines.append(indent)
        else:
            lines[-1] += ' :math:`' + contents[1:-1] + '` '
        return

    elif tag == 'itemizedlist':
        memory = dict(memory)
        memory['list_item_prefix'] = '*  '
        if lines[-1].strip():
            lines.append(indent)
        postfix_lines.append(indent)

    elif tag == 'orderedlist':
        memory = dict(memory)
        memory['list_item_prefix'] = '#. '
        if lines[-1].strip():
            lines.append(indent)
        postfix_lines.append(indent)

    elif tag == 'listitem':
        memory = dict(memory); memory[element] = 1
        item = description_to_string(element, indent + '   ', skipelems, memory)
        lines.append(indent + memory['list_item_prefix'] + item)
        return

    elif tag == 'parameterlist':
        # We parse these separately in the parameter reading process
        return

    elif tag == 'simplesect' and element.get('kind', '') == 'return':
        # We parse these separately in the return-type reading process
        if memory.get('skip_simplesect', True):
            return

    else:
        NOT_IMPLEMENTED_ELEMENTS.add(tag)
        lines.append(ET.tostring(element))

    def add_text(text):
        if text is not None and text.strip():
            tl = text.split('\n')
            lines[-1] += tl[0]
            lines.extend([indent + line for line in tl[1:]])
            if text.endswith('\n'):
                lines.append(indent)

    add_text(element.text)

    for child in children:
        description_to_rst(child, lines, indent, skipelems, memory)
        add_text(child.tail)

    if postfix:
        lines[-1] += postfix

    if postfix_lines:
        lines.extend(postfix_lines)


def xml_table_to_rst(table, indent):
    """
    Read an XML table element and produce a ReStructuredText table

    =====  ====
    Col A  B
    =====  ====
    hi     1.00
    there  3.14
    =====  ====
    """
    rows = int(table.get('rows'))
    cols = int(table.get('cols'))
    data = [[''] * cols for row in range(rows)]
    widths = [0] * cols
    for i, row in enumerate(table):
        for j, entry in enumerate(row):
            memory = {entry: 1}
            text = description_to_string(entry, indent='',
                                         skipelems=('para',),
                                         memory=memory)
            widths[j] = max(widths[j], len(text))
            data[i][j] = text

    ret = [indent] * (rows + 3)
    for w in widths:
        ret[0] += '=' * w + '  '
        ret[2] += '=' * w + '  '
        ret[-1] += '=' * w + '  '

    for i, row in enumerate(data):
        I = 1 if i == 0 else i + 2
        for j, text in enumerate(row):
            ret[I] += text + ' ' * (widths[j] + 2 - len(text))

    return ret


def get_single_element(parent, name, allow_none=False):
    """
    Helper to get one unique child element
    """
    elems = parent.findall(name)
    N = len(elems)
    if N != 1:
        if allow_none and N == 0:
            return None
        raise ValueError('Expected one element %r below %r, got %r' %
                         (name, parent, len(elems)))
    return elems[0]


def findall_recursive(parent, name):
    for item in parent.findall(name):
        yield item
    for child in parent:
        for item in findall_recursive(child, name):
            yield item


def read_doxygen_xml_files(xml_directory, namespace_names, verbose=True):
    """
    Read doxygen XML files from the given directory. Restrict the returned
    namespaces to the ones listed in the namespaces input iterable

    Remember: we are built for speed, not ultimate flexibility, hence the
    restrictions to avoid parsing more than we are interested in actually
    outputing in the end
    """
    if verbose: print('Parsing doxygen XML files in %s' % xml_directory)

    root_namespace = Namespace('')
    for nn in namespace_names:
        root_namespace.add(Namespace(nn))

    # Loop through xml files of compounds and get class definitions
    xml_files = os.listdir(xml_directory)
    for xml_file_name in xml_files:
        if not xml_file_name.startswith('class'):
            continue

        path = os.path.join(xml_directory, xml_file_name)
        try:
            root = ET.parse(path).getroot()
        except Exception as e:
            print('ERROR parsing doxygen xml document', path)
            print(e)
            continue

        compounds = root.findall('compounddef')
        for c in compounds:
            kind = c.attrib['kind']
            names = c.findall('compoundname')

            assert len(names) == 1
            name = names[0].text
            nn = name.split('::')[0]
            namespace = root_namespace.subspaces.get(nn, None)
            if not namespace:
                continue

            item = NamespaceMember.from_compounddef(c, name, kind, xml_file_name)
            namespace.add(item)

            if verbose: print(end='.')
    if verbose: print('DONE\nParsing namespace files:')

    # Loop through other elements in the namespaces
    for namespace in root_namespace.subspaces.values():
        file_name = 'namespace%s.xml' % namespace.name
        path = os.path.join(xml_directory, file_name)
        try:
            root = ET.parse(path).getroot()
        except Exception as e:
            print('ERROR parsing doxygen xml document', path)
            print(e)
            continue

        compound = get_single_element(root, 'compounddef')
        sections = compound.findall('sectiondef')
        for s in sections:
            members = s.findall('memberdef')
            for m in members:
                name = get_single_element(m, 'name').text
                kind = m.attrib['kind']
                item = NamespaceMember.from_memberdef(m, name, kind, xml_file_name, namespace)
                namespace.add(item)
        if verbose: print(' - ', namespace.name)
    if verbose: print('Done parsing files')

    return root_namespace.subspaces


if __name__ == '__main__':
    if len(sys.argv) != 3:
        print('Call me like "script path_to/xml/dir namespace"')
        print('An exampe:\n\tpython parse_doxygen.py doxygen/xml dolfin')
        print('ERROR: I need two arguments!')
        exit(1)

    xml_directory = sys.argv[1]
    namespace = sys.argv[2]

    # Parse the XML files
    namespaces = read_doxygen_xml_files(xml_directory, [namespace])

    # Get sorted list of members
    members = list(namespaces[namespace].members.values())
    members.sort(key=lambda m: m.name)

    # Make Sphinx documentation
    with open('api.rst', 'wt') as out:
        for member in members:
            out.write(member.to_rst())
            out.write('\n')
        out.write('\n')

    # Make SWIG interface file
    with open('docstrings.i', 'wt') as out:
        out.write('// SWIG docstrings generated by doxygen and parse_doxygen.py\n\n')
        for member in members:
            out.write(member.to_swig())
            out.write('\n')
        out.write('\n')

    for tag in NOT_IMPLEMENTED_ELEMENTS:
        print('WARNING: doxygen XML tag %s is not supported by the parser' % tag)



"Unit tests for ghosted meshes"

# Copyright (C) 2016 Garth N. Wells
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function
import pytest
import numpy
from dolfin import *
import os

from dolfin_utils.test import fixture, skip_in_parallel, xfail_in_parallel, cd_tempdir, pushpop_parameters


# See https://bitbucket.org/fenics-project/dolfin/issues/579
def xtest_ghost_vertex_1d(pushpop_parameters):
    parameters["ghost_mode"] = "shared_vertex"
    mesh = UnitIntervalMesh(20)
    #print("Test: {}".format(MPI.sum(mesh.mpi_comm(), mesh.num_cells())))


def xtest_ghost_facet_1d(pushpop_parameters):
    parameters["ghost_mode"] = "shared_facet"
    mesh = UnitIntervalMesh(20)


def test_ghost_2d(pushpop_parameters):
    modes = ["shared_vertex", "shared_facet"]
    for mode in modes:
        parameters["ghost_mode"] = mode
        N = 8
        num_cells = 128

        mesh = UnitSquareMesh(N, N)
        if MPI.size(mesh.mpi_comm()) > 1:
            assert MPI.sum(mesh.mpi_comm(), mesh.num_cells()) > num_cells

        parameters["reorder_cells_gps"] = True
        parameters["reorder_vertices_gps"] = False
        mesh = UnitSquareMesh(N, N)
        if MPI.size(mesh.mpi_comm()) > 1:
            assert MPI.sum(mesh.mpi_comm(), mesh.num_cells()) > num_cells

        parameters["reorder_cells_gps"] = True
        parameters["reorder_vertices_gps"] = True
        mesh = UnitSquareMesh(N, N)
        if MPI.size(mesh.mpi_comm()) > 1:
            assert MPI.sum(mesh.mpi_comm(), mesh.num_cells()) > num_cells

        parameters["reorder_cells_gps"] = False
        parameters["reorder_vertices_gps"] = True
        mesh = UnitSquareMesh(N, N)
        if MPI.size(mesh.mpi_comm()) > 1:
            assert MPI.sum(mesh.mpi_comm(), mesh.num_cells()) > num_cells


def test_ghost_3d(pushpop_parameters):
    modes = ["shared_vertex", "shared_facet"]
    for mode in modes:
        parameters["ghost_mode"] = mode
        N = 2
        num_cells = 48

        mesh = UnitCubeMesh(N, N, N)
        if MPI.size(mesh.mpi_comm()) > 1:
            assert MPI.sum(mesh.mpi_comm(), mesh.num_cells()) > num_cells

        parameters["reorder_cells_gps"] = True
        parameters["reorder_vertices_gps"] = False
        mesh = UnitCubeMesh(N, N, N)
        if MPI.size(mesh.mpi_comm()) > 1:
            assert MPI.sum(mesh.mpi_comm(), mesh.num_cells()) > num_cells

        parameters["reorder_cells_gps"] = True
        parameters["reorder_vertices_gps"] = True
        mesh = UnitCubeMesh(N, N, N)
        if MPI.size(mesh.mpi_comm()) > 1:
            assert MPI.sum(mesh.mpi_comm(), mesh.num_cells()) > num_cells

        parameters["reorder_cells_gps"] = False
        parameters["reorder_vertices_gps"] = True
        mesh = UnitCubeMesh(N, N, N)
        if MPI.size(mesh.mpi_comm()) > 1:
            assert MPI.sum(mesh.mpi_comm(), mesh.num_cells()) > num_cells



# Code snippets for autogeneration of SWIG code
#
# Copyright (C) 2012-2016 Johan Hake
#
# This file is part of DOLFIN.
#
# DOLFIN is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# DOLFIN is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.

import sys

copyright_statement = r"""%(comment)s Auto generated SWIG file for Python interface of DOLFIN
%(comment)s
%(comment)s Copyright (C) 2012-2016 %(holder)s
%(comment)s
%(comment)s This file is part of DOLFIN.
%(comment)s
%(comment)s DOLFIN is free software: you can redistribute it and/or modify
%(comment)s it under the terms of the GNU Lesser General Public License as published by
%(comment)s the Free Software Foundation, either version 3 of the License, or
%(comment)s (at your option) any later version.
%(comment)s
%(comment)s DOLFIN is distributed in the hope that it will be useful,
%(comment)s but WITHOUT ANY WARRANTY; without even the implied warranty of
%(comment)s MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
%(comment)s GNU Lesser General Public License for more details.
%(comment)s
%(comment)s You should have received a copy of the GNU Lesser General Public License
%(comment)s along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
%(comment)s

"""

# Template code for all combined SWIG modules
module_template = r"""
// The PyDOLFIN extension module for the %(module)s module
%%module(package="dolfin.cpp", directors="1") %(module)s
// Define module name for conditional includes
#define %(MODULE)sMODULE

%%{
%(headers)s

// NumPy includes
#define PY_ARRAY_UNIQUE_SYMBOL PyDOLFIN_%(MODULE)s
#include <numpy/arrayobject.h>
%%}

%%init%%{
import_array();
%%}

// Include global SWIG interface files:
// Typemaps, shared_ptr declarations, exceptions, version
%%include "dolfin/swig/globalincludes.i"
%(imports)s

// Turn on SWIG generated signature documentation and include doxygen
// generated docstrings
//%%feature("autodoc", "1");
%(docstrings)s
%(includes)s

"""


if sys.version_info[0] < 3:
    swig_py_args = ""
else:
    swig_py_args = "\n  -py3 \n  -relativeimport"


swig_cmakelists_str = \
"""# Automatic get the module name
get_filename_component(SWIG_MODULE_NAME ${CMAKE_CURRENT_BINARY_DIR} NAME)

# Set project name
project(${SWIG_MODULE_NAME})

# Get dolfin definitions
get_target_property(DOLFIN_DEFINITIONS dolfin INTERFACE_COMPILE_DEFINITIONS)
set(CXX_DEFS)
foreach(_def ${DOLFIN_DEFINITIONS})
    list(APPEND CXX_DEFS "-D${_def}")
endforeach()

# Set SWIG flags
set(CMAKE_SWIG_FLAGS
  -module ${SWIG_MODULE_NAME}
  -shadow
  -modern
  -modernargs
  -fastdispatch
  -fvirtual
  -nosafecstrings
  -noproxydel
  -fastproxy
  -fastinit
  -fastunpack
  -fastquery
  -nobuildnone%s
  -Iinclude/swig
  ${CXX_DEFS}
  ${DOLFIN_PYTHON_DEFINITIONS}
  )

# Get all SWIG interface files
file(READ ${CMAKE_CURRENT_BINARY_DIR}/dependencies.txt DOLFIN_SWIG_DEPENDENCIES )

# This prevents swig being run unnecessarily
set_property(SOURCE module.i PROPERTY SWIG_MODULE_NAME ${SWIG_MODULE_NAME})

# Tell CMake SWIG has generated a C++ file
set_property(SOURCE module.i PROPERTY CPLUSPLUS ON)

# Generate SWIG files in
set(CMAKE_SWIG_OUTDIR ${CMAKE_CURRENT_BINARY_DIR})

# Tell CMake which SWIG interface files should be checked for changes
# when recompile
set(SWIG_MODULE_${SWIG_MODULE_NAME}_EXTRA_DEPS copy_swig_files ${DOLFIN_SWIG_DEPENDENCIES})

# Tell CMake to run SWIG on module.i and to link against libdolfin
if (CMAKE_VERSION VERSION_LESS "3.8")
  swig_add_module(${SWIG_MODULE_NAME} python module.i)
else()
  swig_add_library(${SWIG_MODULE_NAME} LANGUAGE python SOURCES module.i)
endif()

swig_link_libraries(${SWIG_MODULE_NAME} dolfin ${PYTHON_LIBRARIES})

# Install Python targets and .py files
install(TARGETS
  ${SWIG_MODULE_${SWIG_MODULE_NAME}_REAL_NAME}
  DESTINATION ${DOLFIN_INSTALL_PYTHON_MODULE_DIR}/dolfin/cpp
  COMPONENT RuntimeLibraries
  )
install(FILES
  ${CMAKE_CURRENT_BINARY_DIR}/${SWIG_MODULE_NAME}.py
  DESTINATION ${DOLFIN_INSTALL_PYTHON_MODULE_DIR}/dolfin/cpp
  COMPONENT RuntimeLibraries
  )
""" % (swig_py_args,)



# -*- coding: utf-8 -*-
"""The compilemodules module of dolfin"""
from dolfin.compilemodules import compilemodule
from dolfin.compilemodules import expressions
from dolfin.compilemodules import subdomains
from dolfin.compilemodules import jit

from .compilemodule import compile_extension_module
from .expressions import compile_expressions
from .subdomains import CompiledSubDomain
from .jit import jit

# NOTE: The automatic documentation system in DOLFIN requires to _not_ define
# NOTE: classes or functions within this file. Use separate modules for that
# NOTE: purpose.

__all__ = ["compile_extension_module", "compile_expressions",
           "CompiledSubDomain", "jit"]
